#!/usr/bin/env python

# Copyright (c) 2016 Ericsson AB and others.
#
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Apache License, Version 2.0
# which accompanies this distribution, and is available at
# http://www.apache.org/licenses/LICENSE-2.0

"""TierBuilder class to parse testcases config file"""

import re
import yaml

from xtesting.ci import tier_handler
from xtesting.utils import env


class TierBuilder():
    # pylint: disable=missing-docstring

    def __init__(self, testcases_file):
        self.ci_installer = env.get('INSTALLER_TYPE')
        self.ci_scenario = env.get('DEPLOY_SCENARIO')
        self.testcases_file = testcases_file
        self.dic_tier_array = None
        self.tier_objects = []
        self.testcases_yaml = None
        self.generate_tiers()

    def read_test_yaml(self):
        with open(self.testcases_file, encoding='utf-8') as tc_file:
            self.testcases_yaml = yaml.safe_load(tc_file)

        self.dic_tier_array = []
        for tier in self.testcases_yaml.get("tiers"):
            self.dic_tier_array.append(tier)

    def generate_tiers(self):
        if self.dic_tier_array is None:
            self.read_test_yaml()

        del self.tier_objects[:]
        for dic_tier in self.dic_tier_array:
            tier = tier_handler.Tier(
                name=dic_tier['name'],
                description=dic_tier.get('description', ''))
            for dic_testcase in dic_tier['testcases']:
                testcase = tier_handler.TestCase(
                    name=dic_testcase['case_name'],
                    enabled=dic_testcase.get('enabled', True),
                    skipped=False,
                    criteria=dic_testcase.get('criteria', 100),
                    blocking=dic_testcase.get('blocking', True),
                    description=dic_testcase.get('description', ''),
                    project=dic_testcase['project_name'])
                if not dic_testcase.get('dependencies'):
                    if testcase.is_enabled():
                        tier.add_test(testcase)
                    else:
                        testcase.skipped = True
                        tier.skip_test(testcase)
                else:
                    for dependency in dic_testcase['dependencies']:
                        kenv = list(dependency.keys())[0]
                        if not re.search(dependency[kenv],
                                         env.get(kenv) or ''):
                            testcase.skipped = True
                            tier.skip_test(testcase)
                            break
                    else:
                        if testcase.is_enabled():
                            tier.add_test(testcase)
                        else:
                            testcase.skipped = True
                            tier.skip_test(testcase)
            self.tier_objects.append(tier)

    def get_tiers(self):
        return self.tier_objects

    def get_tier_names(self):
        tier_names = []
        for tier in self.tier_objects:
            tier_names.append(tier.get_name())
        return tier_names

    def get_tier(self, tier_name):
        for i, _ in enumerate(self.tier_objects):
            if self.tier_objects[i].get_name() == tier_name:
                return self.tier_objects[i]
        return None

    def get_tier_name(self, test_name):
        for i, _ in enumerate(self.tier_objects):
            if self.tier_objects[i].is_test(test_name):
                return self.tier_objects[i].name
        return None

    def get_test(self, test_name):
        for i, _ in enumerate(self.tier_objects):
            if self.tier_objects[i].is_test(test_name):
                return self.tier_objects[i].get_test(test_name)
        return None

    def get_tests(self, tier_name):
        for i, _ in enumerate(self.tier_objects):
            if self.tier_objects[i].get_name() == tier_name:
                return self.tier_objects[i].get_tests()
        return None

    def __str__(self):
        output = ""
        for i, _ in enumerate(self.tier_objects):
            output += str(self.tier_objects[i]) + "\n"
        return output



#!/usr/bin/env python

# Copyright (c) 2021 Orange and others.
#
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Apache License, Version 2.0
# which accompanies this distribution, and is available at
# http://www.apache.org/licenses/LICENSE-2.0

# pylint: disable=missing-docstring

import logging
import unittest

import mock
import munch

from xtesting.core import ansible


class RunTesting(unittest.TestCase):

    def setUp(self):
        self.test = ansible.Ansible()

    @mock.patch("shutil.which", return_value=None)
    def test_check1(self, which):
        self.test.check_requirements()
        self.assertEqual(self.test.is_skipped, True)
        which.assert_called_once_with("ansible-playbook")

    @mock.patch("shutil.which", return_value='/usr/bin/ansible-playbook')
    def test_check2(self, which):
        self.test.check_requirements()
        self.assertEqual(self.test.is_skipped, False)
        which.assert_called_once_with("ansible-playbook")

    @mock.patch("os.path.isdir", return_value=False)
    def test_fail1(self, isdir):
        self.assertEqual(self.test.run(), self.test.EX_RUN_ERROR)
        isdir.assert_not_called()

    @mock.patch("os.path.isdir", return_value=False)
    def test_fail2(self, isdir):
        private_data_dir = "titi"
        self.assertEqual(
            self.test.run(private_data_dir=private_data_dir),
            self.test.EX_RUN_ERROR)
        isdir.assert_called_once_with(private_data_dir)

    @mock.patch("ansible_runner.run", side_effect=Exception)
    @mock.patch("os.makedirs")
    @mock.patch("os.path.exists", return_value=True)
    @mock.patch("os.path.isdir", return_value=True)
    def test_fail3(self, *args):
        private_data_dir = "titi"
        self.assertEqual(
            self.test.run(private_data_dir=private_data_dir),
            self.test.EX_RUN_ERROR)
        args[0].assert_called_once_with(private_data_dir)
        args[1].assert_called_once_with(self.test.res_dir)
        args[2].assert_not_called()
        args[3].assert_called_with(
            private_data_dir=private_data_dir, quiet=True,
            artifact_dir=self.test.res_dir)

    @mock.patch("ansible_runner.run", side_effect=Exception)
    @mock.patch("os.makedirs")
    @mock.patch("os.path.exists", return_value=False)
    @mock.patch("os.path.isdir", return_value=True)
    def test_fail4(self, *args):
        private_data_dir = "titi"
        self.assertEqual(
            self.test.run(private_data_dir=private_data_dir),
            self.test.EX_RUN_ERROR)
        args[0].assert_called_once_with(private_data_dir)
        args[1].assert_called_once_with(self.test.res_dir)
        args[2].assert_called_once_with(self.test.res_dir)
        args[3].assert_called_with(
            private_data_dir=private_data_dir, quiet=True,
            artifact_dir=self.test.res_dir)

    @mock.patch("ansible_runner.run")
    @mock.patch("os.makedirs", side_effect=Exception)
    @mock.patch("os.path.exists", return_value=False)
    @mock.patch("os.path.isdir", return_value=True)
    def test_fail5(self, *args):
        private_data_dir = "titi"
        self.assertEqual(
            self.test.run(private_data_dir=private_data_dir),
            self.test.EX_RUN_ERROR)
        args[0].assert_called_once_with(private_data_dir)
        args[1].assert_called_once_with(self.test.res_dir)
        args[2].assert_called_once_with(self.test.res_dir)
        args[3].assert_not_called()

    @mock.patch("ansible_runner.run", return_value={})
    @mock.patch("os.makedirs")
    @mock.patch("os.path.exists", return_value=False)
    @mock.patch("os.path.isdir", return_value=True)
    def test_fail6(self, *args):
        private_data_dir = "titi"
        self.assertEqual(
            self.test.run(
                private_data_dir=private_data_dir, quiet=False,
                artifact_dir="overridden"),
            self.test.EX_RUN_ERROR)
        args[0].assert_called_once_with(private_data_dir)
        args[1].assert_called_once_with(self.test.res_dir)
        args[2].assert_called_once_with(self.test.res_dir)
        args[3].assert_called_with(
            private_data_dir=private_data_dir, quiet=False,
            artifact_dir=self.test.res_dir)

    @mock.patch("ansible_runner.run",
                return_value=munch.Munch(rc=0, stats={"foo": "bar"}))
    @mock.patch("os.makedirs")
    @mock.patch("os.path.exists", return_value=False)
    @mock.patch("os.path.isdir", return_value=True)
    def test_res_ok1(self, *args):
        private_data_dir = "titi"
        self.assertEqual(
            self.test.run(
                private_data_dir=private_data_dir, quiet=False,
                artifact_dir="overridden"),
            self.test.EX_OK)
        args[0].assert_called_once_with(private_data_dir)
        args[1].assert_called_once_with(self.test.res_dir)
        args[2].assert_called_once_with(self.test.res_dir)
        args[3].assert_called_with(
            private_data_dir=private_data_dir, quiet=False,
            artifact_dir=self.test.res_dir)
        self.assertEqual(self.test.is_successful(), self.test.EX_OK)
        self.assertEqual(self.test.details, {"foo": "bar"})

    @mock.patch("ansible_runner.run",
                return_value=munch.Munch(rc=0, stats={"foo": "bar"}))
    @mock.patch("os.makedirs")
    @mock.patch("os.path.exists", return_value=False)
    @mock.patch("os.path.isdir", return_value=True)
    def test_res_ok2(self, *args):
        private_data_dir = "titi"
        self.assertEqual(
            self.test.run(
                private_data_dir=private_data_dir, quiet=True,
                artifact_dir="overridden"),
            self.test.EX_OK)
        args[0].assert_called_once_with(private_data_dir)
        args[1].assert_called_once_with(self.test.res_dir)
        args[2].assert_called_once_with(self.test.res_dir)
        args[3].assert_called_with(
            private_data_dir=private_data_dir, quiet=True,
            artifact_dir=self.test.res_dir)
        self.assertEqual(self.test.is_successful(), self.test.EX_OK)
        self.assertEqual(self.test.details, {"foo": "bar"})

    @mock.patch("ansible_runner.run",
                return_value=munch.Munch(rc=0, stats={"foo": "bar"}))
    @mock.patch("os.makedirs")
    @mock.patch("os.path.exists", return_value=False)
    @mock.patch("os.path.isdir", return_value=True)
    def test_res_ok3(self, *args):
        private_data_dir = "titi"
        self.assertEqual(
            self.test.run(
                private_data_dir=private_data_dir, artifact_dir="overridden"),
            self.test.EX_OK)
        args[0].assert_called_once_with(private_data_dir)
        args[1].assert_called_once_with(self.test.res_dir)
        args[2].assert_called_once_with(self.test.res_dir)
        args[3].assert_called_with(
            private_data_dir=private_data_dir, quiet=True,
            artifact_dir=self.test.res_dir)
        self.assertEqual(self.test.is_successful(), self.test.EX_OK)
        self.assertEqual(self.test.details, {"foo": "bar"})

    @mock.patch("ansible_runner.run",
                return_value=munch.Munch(rc=1, stats={"foo": "bar"}))
    @mock.patch("os.makedirs")
    @mock.patch("os.path.exists", return_value=False)
    @mock.patch("os.path.isdir", return_value=True)
    def test_res_ko(self, *args):
        private_data_dir = "titi"
        self.assertEqual(
            self.test.run(
                private_data_dir=private_data_dir, artifact_dir="overridden"),
            self.test.EX_OK)
        args[0].assert_called_once_with(private_data_dir)
        args[1].assert_called_once_with(self.test.res_dir)
        args[2].assert_called_once_with(self.test.res_dir)
        args[3].assert_called_with(
            private_data_dir=private_data_dir, quiet=True,
            artifact_dir=self.test.res_dir)
        self.assertEqual(self.test.is_successful(),
                         self.test.EX_TESTCASE_FAILED)
        self.assertEqual(self.test.details, {"foo": "bar"})


if __name__ == "__main__":
    logging.disable(logging.CRITICAL)
    unittest.main(verbosity=2)



#!/usr/bin/env python

# Copyright (c) 2016 Orange and others.
#
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Apache License, Version 2.0
# which accompanies this distribution, and is available at
# http://www.apache.org/licenses/LICENSE-2.0

# pylint: disable=missing-docstring

import time

from xtesting.core import testcase


class Test(testcase.TestCase):

    def run(self, **kwargs):
        self.start_time = time.time()
        print("Hello World")
        self.result = 100
        self.stop_time = time.time()



#!/usr/bin/env python

# Copyright (c) 2019 Orange and others.
#
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Apache License, Version 2.0
# which accompanies this distribution, and is available at
# http://www.apache.org/licenses/LICENSE-2.0

"""Dump DB and artifacts for third-party certifications."""

import json
import logging
import logging.config
import mimetypes
import os
import re
import zipfile

from urllib.parse import urlparse
import boto3
from boto3.s3.transfer import TransferConfig
import botocore
import pkg_resources
import requests

from xtesting.core import testcase
from xtesting.utils import env
from xtesting.utils import constants

__author__ = "Cedric Ollivier <cedric.ollivier@orange.com>"


class Campaign():
    "Dump, archive and publish all results and artifacts from a campaign."

    EX_OK = os.EX_OK
    """everything is OK"""

    EX_DUMP_FROM_DB_ERROR = os.EX_SOFTWARE - 5
    """dump_db() failed"""

    EX_DUMP_ARTIFACTS_ERROR = os.EX_SOFTWARE - 6
    """dump_artifacts() failed"""

    EX_ZIP_CAMPAIGN_FILES_ERROR = os.EX_SOFTWARE - 7
    """dump_artifacts() failed"""

    __logger = logging.getLogger(__name__)

    @staticmethod
    def dump_db():
        """Dump all test campaign results from the DB.

        It allows collecting all the results from the DB.

        It could be overriden if the common implementation is not
        suitable.

        The next vars must be set in env:

            * TEST_DB_URL,
            * BUILD_TAG.

        Returns:
            Campaign.EX_OK if results were collected from DB.
            Campaign.EX_DUMP_FROM_DB_ERROR otherwise.
        """
        try:
            url = env.get('TEST_DB_URL')
            req = requests.get(
                f"{url}?build_tag={env.get('BUILD_TAG')}",
                headers=testcase.TestCase.headers)
            req.raise_for_status()
            output = req.json()
            Campaign.__logger.debug("data from DB: \n%s", output)
            for i, _ in enumerate(output["results"]):
                for j, _ in enumerate(
                        output["results"][i]["details"]["links"]):
                    output["results"][i]["details"]["links"][j] = re.sub(
                        "^{os.environ['HTTP_DST_URL']}/*", '',
                        output["results"][i]["details"]["links"][j])
            Campaign.__logger.debug("data to archive: \n%s", output)
            with open("{env.get('BUILD_TAG')}.json", "w",
                      encoding='utf-8') as dfile:
                json.dump(output, dfile)
        except Exception:  # pylint: disable=broad-except
            Campaign.__logger.exception(
                "The results cannot be collected from DB")
            return Campaign.EX_DUMP_FROM_DB_ERROR
        return Campaign.EX_OK

    @staticmethod
    def dump_artifacts():
        """Dump all test campaign artifacts from the S3 repository.

        It allows collecting all the artifacts from the S3 repository.

        It could be overriden if the common implementation is not
        suitable.

        The credentials must be configured before publishing the artifacts:

            * fill ~/.aws/credentials or ~/.boto,
            * set AWS_ACCESS_KEY_ID and AWS_SECRET_ACCESS_KEY in env.

        The next vars must be set in env:

            * S3_ENDPOINT_URL (http://127.0.0.1:9000),
            * S3_DST_URL (s3://xtesting/prefix),

        Returns:
            Campaign.EX_OK if artifacts were published to repository.
            Campaign.EX_DUMP_ARTIFACTS_ERROR otherwise.
        """
        try:
            build_tag = env.get('BUILD_TAG')
            b3resource = boto3.resource(
                's3', endpoint_url=os.environ["S3_ENDPOINT_URL"])
            dst_s3_url = os.environ["S3_DST_URL"]
            multipart_threshold = 5 * 1024 ** 5 if "google" in os.environ[
                "S3_ENDPOINT_URL"] else 8 * 1024 * 1024
            config = TransferConfig(multipart_threshold=multipart_threshold)
            bucket_name = urlparse(dst_s3_url).netloc
            s3path = re.search(
                '^/*(.*)/*$', urlparse(dst_s3_url).path).group(1)
            prefix = os.path.join(s3path, build_tag)
            # pylint: disable=no-member
            for s3_object in b3resource.Bucket(bucket_name).objects.filter(
                    Prefix=f"{prefix}/"):
                path, _ = os.path.split(s3_object.key)
                lpath = re.sub(f'^{s3path}/*', '', path)
                if lpath and not os.path.exists(lpath):
                    os.makedirs(lpath)
                # pylint: disable=no-member
                b3resource.Bucket(bucket_name).download_file(
                    s3_object.key,
                    re.sub(f'^{s3path}/*', '', s3_object.key),
                    Config=config)
                Campaign.__logger.info(
                    "Downloading %s",
                    re.sub(f'^{s3path}/*', '', s3_object.key))
            return Campaign.EX_OK
        except Exception:  # pylint: disable=broad-except
            Campaign.__logger.exception("Cannot publish the artifacts")
            return Campaign.EX_DUMP_ARTIFACTS_ERROR

    @staticmethod
    def zip_campaign_files():  # pylint: disable=too-many-locals
        """Archive and publish all test campaign data to the S3 repository.

        It allows collecting all the artifacts from the S3 repository.

        It could be overriden if the common implementation is not
        suitable.

        The credentials must be configured before publishing the artifacts:

            * fill ~/.aws/credentials or ~/.boto,
            * set AWS_ACCESS_KEY_ID and AWS_SECRET_ACCESS_KEY in env.

        The next vars must be set in env:

            * S3_ENDPOINT_URL (http://127.0.0.1:9000),
            * S3_DST_URL (s3://xtesting/prefix),

        Returns:
            Campaign.EX_OK if artifacts were published to repository.
            Campaign.EX_DUMP_ARTIFACTS_ERROR otherwise.
        """
        try:
            build_tag = env.get('BUILD_TAG')
            assert Campaign.dump_db() == Campaign.EX_OK
            assert Campaign.dump_artifacts() == Campaign.EX_OK
            with zipfile.ZipFile(f'{build_tag}.zip',
                                 'w', zipfile.ZIP_DEFLATED) as zfile:
                zfile.write(f"{build_tag}.json")
                for root, _, files in os.walk(build_tag):
                    for filename in files:
                        zfile.write(os.path.join(root, filename))
            b3resource = boto3.resource(
                's3', endpoint_url=os.environ["S3_ENDPOINT_URL"])
            dst_s3_url = os.environ["S3_DST_URL"]
            multipart_threshold = 5 * 1024 ** 5 if "google" in os.environ[
                "S3_ENDPOINT_URL"] else 8 * 1024 * 1024
            config = TransferConfig(multipart_threshold=multipart_threshold)
            bucket_name = urlparse(dst_s3_url).netloc
            mime_type = mimetypes.guess_type(f'{build_tag}.zip')
            path = urlparse(dst_s3_url).path.strip("/")
            # pylint: disable=no-member
            b3resource.Bucket(bucket_name).upload_file(
                f'{build_tag}.zip',
                os.path.join(path, f'{build_tag}.zip'),
                Config=config,
                ExtraArgs={'ContentType': mime_type[
                    0] or 'application/octet-stream'})
            dst_http_url = os.environ["HTTP_DST_URL"]
            link = os.path.join(dst_http_url, f'{build_tag}.zip')
            Campaign.__logger.info(
                "All data were successfully published:\n\n%s", link)
            return Campaign.EX_OK
        except KeyError as ex:
            Campaign.__logger.error("Please check env var: %s", str(ex))
            return Campaign.EX_ZIP_CAMPAIGN_FILES_ERROR
        except botocore.exceptions.NoCredentialsError:
            Campaign.__logger.error(
                "Please fill ~/.aws/credentials, ~/.boto or set "
                "AWS_ACCESS_KEY_ID and AWS_SECRET_ACCESS_KEY in env")
            return Campaign.EX_ZIP_CAMPAIGN_FILES_ERROR
        except Exception:  # pylint: disable=broad-except
            Campaign.__logger.exception("Cannot publish the artifacts")
            return Campaign.EX_ZIP_CAMPAIGN_FILES_ERROR


def main():
    """Entry point for Campaign.zip_campaign_files()."""
    if not os.path.exists(testcase.TestCase.dir_results):
        os.makedirs(testcase.TestCase.dir_results)
    if env.get('DEBUG').lower() == 'true':
        logging.config.fileConfig(pkg_resources.resource_filename(
            'xtesting', constants.DEBUG_INI_PATH))
    else:
        logging.config.fileConfig(pkg_resources.resource_filename(
            'xtesting', constants.INI_PATH))
    logging.captureWarnings(True)
    Campaign.zip_campaign_files()



#!/usr/bin/env python

# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Apache License, Version 2.0
# which accompanies this distribution, and is available at
# http://www.apache.org/licenses/LICENSE-2.0

# pylint: disable=missing-docstring

import logging
import os
import unittest

import mock

from xtesting.ci import tier_builder


class TierBuilderTesting(unittest.TestCase):

    def setUp(self):
        self.testcase = {
            'enabled': True,
            'case_name': 'test_name', 'criteria': 'test_criteria',
            'blocking': 'test_blocking', 'description': 'test_desc',
            'project_name': 'project_name'}
        self.testcase_disabled = {
            'enabled': False,
            'case_name': 'test_name_disabled', 'criteria': 'test_criteria',
            'blocking': 'test_blocking', 'description': 'test_desc',
            'project_name': 'project_name'}
        self.dic_tier = {
            'name': 'test_tier', 'description': 'test_desc',
            'testcases': [self.testcase, self.testcase_disabled]}
        self.mock_yaml = mock.Mock()
        attrs = {'get.return_value': [self.dic_tier]}
        self.mock_yaml.configure_mock(**attrs)

        with mock.patch('xtesting.ci.tier_builder.yaml.safe_load',
                        return_value=self.mock_yaml), \
                mock.patch('builtins.open', mock.mock_open()):
            os.environ["INSTALLER_TYPE"] = 'test_installer'
            os.environ["DEPLOY_SCENARIO"] = 'test_scenario'
            self.tierbuilder = tier_builder.TierBuilder('testcases_file')
        self.tier_obj = self.tierbuilder.tier_objects[0]

    def test_get_tiers(self):
        self.assertEqual(self.tierbuilder.get_tiers(),
                         [self.tier_obj])

    def test_get_tier_names(self):
        self.assertEqual(self.tierbuilder.get_tier_names(),
                         ['test_tier'])

    def test_get_tier_present_tier(self):
        self.assertEqual(self.tierbuilder.get_tier('test_tier'),
                         self.tier_obj)

    def test_get_tier_missing_tier(self):
        self.assertEqual(self.tierbuilder.get_tier('test_tier2'),
                         None)

    def test_get_test_present_test(self):
        self.assertEqual(self.tierbuilder.get_test('test_name'),
                         self.tier_obj.get_test('test_name'))

    def test_get_test_disabled(self):
        self.assertEqual(
            self.tierbuilder.get_test('test_name_disabled'),
            self.tier_obj.get_test('test_name_disabled'))
        self.assertEqual(
            self.tier_obj.get_skipped_test()[0].name, 'test_name_disabled')

    def test_get_test_missing_test(self):
        self.assertEqual(self.tierbuilder.get_test('test_name2'),
                         None)

    def test_get_tests_present_tier(self):
        self.assertEqual(self.tierbuilder.get_tests('test_tier'),
                         self.tier_obj.tests_array)

    def test_get_tests_missing_tier(self):
        self.assertEqual(self.tierbuilder.get_tests('test_tier2'),
                         None)

    def test_get_tier_name_ok(self):
        self.assertEqual(self.tierbuilder.get_tier_name('test_name'),
                         'test_tier')

    def test_get_tier_name_ko(self):
        self.assertEqual(self.tierbuilder.get_tier_name('test_name2'), None)

    def test_str(self):
        message = str(self.tierbuilder)
        self.assertTrue('test_tier' in message)
        self.assertTrue('test_desc' in message)
        self.assertTrue('test_name' in message)


if __name__ == "__main__":
    logging.disable(logging.CRITICAL)
    unittest.main(verbosity=2)



#!/usr/bin/env python

# Copyright (c) 2016 Cable Television Laboratories, Inc. and others.
#
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Apache License, Version 2.0
# which accompanies this distribution, and is available at
# http://www.apache.org/licenses/LICENSE-2.0

"""Define the parent class to run unittest.TestSuite as TestCase."""

from __future__ import division
from io import BytesIO
import logging
import os
import shutil
import subprocess
import time
import unittest

from subunit.run import SubunitTestRunner

from xtesting.core import testcase

__author__ = ("Steven Pisarski <s.pisarski@cablelabs.com>, "
              "Cedric Ollivier <cedric.ollivier@orange.com>")


class Suite(testcase.TestCase):
    """Base model for running unittest.TestSuite."""

    __logger = logging.getLogger(__name__)

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.suite = None

    @classmethod
    def generate_stats(cls, stream):
        """Generate stats from subunit stream

        Raises:
            Exception
        """
        stream.seek(0)
        with subprocess.Popen(
                ['subunit-stats'], stdin=subprocess.PIPE,
                stdout=subprocess.PIPE) as stats:
            output, _ = stats.communicate(stream.read())
            cls.__logger.info("\n\n%s", output.decode("utf-8"))

    def generate_xunit(self, stream):
        """Generate junit report from subunit stream

        Raises:
            Exception
        """
        stream.seek(0)
        with open(f"{self.res_dir}/results.xml", "w", encoding='utf-8') as xml:
            with subprocess.Popen(
                    ['subunit2junitxml'], stdin=subprocess.PIPE,
                    stdout=subprocess.PIPE) as stats:
                output, _ = stats.communicate(stream.read())
                xml.write(output.decode("utf-8"))

    def generate_html(self, stream):
        """Generate html report from subunit stream

        Raises:
            CalledProcessError
        """
        cmd = ['subunit2html', stream, f'{self.res_dir}/results.html']
        output = subprocess.check_output(cmd)
        self.__logger.debug("\n%s\n\n%s", ' '.join(cmd), output)

    def run(self, **kwargs):
        """Run the test suite.

        It allows running any unittest.TestSuite and getting its
        execution status.

        By default, it runs the suite defined as instance attribute.
        It can be overriden by passing name as arg. It must
        conform with TestLoader.loadTestsFromName().

        It sets the following attributes required to push the results
        to DB:

            * result,
            * start_time,
            * stop_time,
            * details.

        Args:
            kwargs: Arbitrary keyword arguments.

        Return:
            TestCase.EX_OK if any TestSuite has been run
            TestCase.EX_RUN_ERROR otherwise.
        """
        try:
            name = kwargs["name"]
            try:
                self.suite = unittest.TestLoader().loadTestsFromName(name)
            except ImportError:
                self.__logger.error("Can not import %s", name)
                return testcase.TestCase.EX_RUN_ERROR
        except KeyError:
            pass
        try:
            assert self.suite
            self.start_time = time.time()
            if not os.path.isdir(self.res_dir):
                os.makedirs(self.res_dir)
            stream = BytesIO()
            result = SubunitTestRunner(
                stream=stream, verbosity=2).run(self.suite).decorated
            self.generate_stats(stream)
            self.generate_xunit(stream)
            with open(f'{self.res_dir}/subunit_stream', 'wb') as subfd:
                stream.seek(0)
                shutil.copyfileobj(stream, subfd)
            self.generate_html(f'{self.res_dir}/subunit_stream')
            self.stop_time = time.time()
            self.details = {
                "testsRun": result.testsRun,
                "failures": len(result.failures),
                "errors": len(result.errors)}
            self.result = 100 * (
                (result.testsRun - (len(result.failures) +
                                    len(result.errors))) /
                result.testsRun)
            return testcase.TestCase.EX_OK
        except AssertionError:
            self.__logger.error("No suite is defined")
            return testcase.TestCase.EX_RUN_ERROR
        except ZeroDivisionError:
            self.__logger.error("No test has been run")
            return testcase.TestCase.EX_RUN_ERROR
        except Exception:  # pylint: disable=broad-except
            self.__logger.exception("something wrong occurs")
            return testcase.TestCase.EX_RUN_ERROR



#!/usr/bin/env python

# Copyright (c) 2016 Orange and others.
#
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Apache License, Version 2.0
# which accompanies this distribution, and is available at
# http://www.apache.org/licenses/LICENSE-2.0

"""Define the parent class of all VNF TestCases."""

import logging
import time
import uuid

from xtesting.core import testcase

__author__ = ("Morgan Richomme <morgan.richomme@orange.com>, "
              "Valentin Boucher <valentin.boucher@orange.com>")


class VnfPreparationException(Exception):
    """Raise when VNF preparation cannot be executed."""


class OrchestratorDeploymentException(Exception):
    """Raise when orchestrator cannot be deployed."""


class VnfDeploymentException(Exception):
    """Raise when VNF cannot be deployed."""


class VnfTestException(Exception):
    """Raise when VNF cannot be tested."""


class VnfOnBoarding(testcase.TestCase):
    # pylint: disable=too-many-instance-attributes
    """Base model for VNF test cases."""

    __logger = logging.getLogger(__name__)

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.uuid = uuid.uuid4()
        self.user_name = f"{self.case_name}-{self.uuid}"
        self.tenant_name = f"{self.case_name}-{self.uuid}"
        self.snaps_creds = {}
        self.created_object = []
        self.os_project = None
        self.tenant_description = (
            f"Created by OPNFV Functest: {self.case_name}")

    def run(self, **kwargs):
        """
        Run of the VNF test case:

            * Deploy an orchestrator if needed (e.g. heat, cloudify, ONAP,...),
            * Deploy the VNF,
            * Perform tests on the VNF

          A VNF test case is successfull when the 3 steps are PASS
          If one of the step is FAIL, the test case is FAIL

        Returns:
          TestCase.EX_OK if result is 'PASS'.
          TestCase.EX_TESTCASE_FAILED otherwise.
        """
        self.start_time = time.time()

        try:
            self.prepare()
            if (self.deploy_orchestrator() and
                    self.deploy_vnf() and
                    self.test_vnf()):
                self.stop_time = time.time()
                # Calculation with different weight depending on the steps TODO
                self.result = 100
                return testcase.TestCase.EX_OK
            self.result = 0
            self.stop_time = time.time()
            return testcase.TestCase.EX_TESTCASE_FAILED
        except Exception:  # pylint: disable=broad-except
            self.stop_time = time.time()
            self.__logger.exception("Exception on VNF testing")
            return testcase.TestCase.EX_TESTCASE_FAILED

    def prepare(self):
        """
        Prepare the environment for VNF testing:

        Returns base.TestCase.EX_OK if preparation is successfull

        Raise VnfPreparationException in case of problem
        """
        self.__logger.exception("VNF must be prepared")
        raise VnfPreparationException

    def deploy_orchestrator(self):
        """
        Deploy an orchestrator (optional).

        If this method is overriden then raise orchestratorDeploymentException
        if error during orchestrator deployment
        """
        self.__logger.info("Deploy orchestrator (if necessary)")
        return True

    def deploy_vnf(self):
        """
        Deploy the VNF

        This function MUST be implemented by vnf test cases.
        The details section MAY be updated in the vnf test cases.

        The deployment can be executed via a specific orchestrator
        or using build-in orchestrators such as heat, OpenBaton, cloudify,
        juju, onap, ...

        Returns:
            True if the VNF is properly deployed
            False if the VNF is not deployed

        Raise VnfDeploymentException if error during VNF deployment
        """
        self.__logger.error("VNF must be deployed")
        raise VnfDeploymentException

    def test_vnf(self):
        """
        Test the VNF

        This function MUST be implemented by vnf test cases.
        The details section MAY be updated in the vnf test cases.

        Once a VNF is deployed, it is assumed that specific test suite can be
        run to validate the VNF.
        Please note that the same test suite can be used on several test case
        (e.g. clearwater test suite can be used whatever the orchestrator used
        for the deployment)

        Returns:
            True if VNF tests are PASS
            False if test suite is FAIL

        Raise VnfTestException if error during VNF test
        """
        self.__logger.error("VNF must be tested")
        raise VnfTestException

    def clean(self):
        """
        Clean VNF test case.

        It is up to the test providers to delete resources used for the tests.
        By default we clean:

            * the user,
            * the tenant
        """
        self.__logger.info('Removing the VNF resources ..')
        for creator in reversed(self.created_object):
            try:
                creator.clean()
            except Exception as exc:  # pylint: disable=broad-except
                self.__logger.error('Unexpected error cleaning - %s', exc)



#!/usr/bin/env python

# Copyright (c) 2017 Orange and others.
#
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Apache License, Version 2.0
# which accompanies this distribution, and is available at
# http://www.apache.org/licenses/LICENSE-2.0

"""Define classes required to run any Robot suites."""

from __future__ import division

import logging
import os

from io import StringIO
import robot.api
from robot.errors import RobotError
from robot.reporting import resultwriter
import robot.run
from robot.utils.robottime import timestamp_to_secs

from xtesting.core import testcase

__author__ = "Cedric Ollivier <cedric.ollivier@orange.com>"


class ResultVisitor(robot.api.ResultVisitor):
    """Visitor to get result details."""

    def __init__(self):
        self._data = []

    def visit_test(self, test):
        output = {}
        output['name'] = test.name
        output['parent'] = test.parent.name
        output['status'] = test.status
        output['starttime'] = test.starttime
        output['endtime'] = test.endtime
        output['text'] = test.message
        output['elapsedtime'] = test.elapsedtime
        self._data.append(output)

    def get_data(self):
        """Get the details of the result."""
        return self._data


class RobotFramework(testcase.TestCase):
    """RobotFramework runner."""

    __logger = logging.getLogger(__name__)

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.xml_file = os.path.join(self.res_dir, 'output.xml')

    def parse_results(self):
        """Parse output.xml and get the details in it."""
        result = robot.api.ExecutionResult(self.xml_file)
        visitor = ResultVisitor()
        result.visit(visitor)
        try:
            self.result = 100 * (
                result.suite.statistics.passed /
                result.suite.statistics.total)
        except ZeroDivisionError:
            self.__logger.error("No test has been run")
        self.start_time = timestamp_to_secs(result.suite.starttime)
        self.stop_time = timestamp_to_secs(result.suite.endtime)
        self.details = {}
        self.details['description'] = result.suite.name
        self.details['tests'] = visitor.get_data()

    def generate_report(self):
        """Generate html and xunit outputs"""
        result = robot.api.ExecutionResult(self.xml_file)
        writer = resultwriter.ResultWriter(result)
        return writer.write_results(
            report=f'{self.res_dir}/report.html',
            log=f'{self.res_dir}/log.html',
            xunit=f'{self.res_dir}/xunit.xml')

    def run(self, **kwargs):
        """Run the RobotFramework suites

        Here are the steps:
           * create the output directories if required,
           * get the results in output.xml,
           * delete temporary files.

        Args:
            kwargs: Arbitrary keyword arguments.

        Returns:
            EX_OK if all suites ran well.
            EX_RUN_ERROR otherwise.
        """
        try:
            suites = kwargs["suites"]
            variable = kwargs.get("variable", [])
            variablefile = kwargs.get("variablefile", [])
            include = kwargs.get("include", [])
        except KeyError:
            self.__logger.exception("Mandatory args were not passed")
            return self.EX_RUN_ERROR
        if not os.path.exists(self.res_dir):
            try:
                os.makedirs(self.res_dir)
            except Exception:  # pylint: disable=broad-except
                self.__logger.exception("Cannot create %s", self.res_dir)
                return self.EX_RUN_ERROR
        stream = StringIO()
        robot.run(*suites, variable=variable, variablefile=variablefile,
                  include=include, output=self.xml_file, log='NONE',
                  report='NONE', stdout=stream)
        self.__logger.info("\n%s", stream.getvalue())
        try:
            self.parse_results()
            self.__logger.info("Results were successfully parsed")
            self.generate_report()
            self.__logger.info("Results were successfully generated")
        except RobotError as ex:
            self.__logger.error("Run suites before publishing: %s", ex.message)
            return self.EX_RUN_ERROR
        except Exception:  # pylint: disable=broad-except
            self.__logger.exception("Cannot parse results")
            return self.EX_RUN_ERROR
        return self.EX_OK



#!/usr/bin/env python

# Copyright (c) 2017 Orange and others.
#
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Apache License, Version 2.0
# which accompanies this distribution, and is available at
# http://www.apache.org/licenses/LICENSE-2.0

"""Define the classes required to fully cover robot."""

import logging
import os
import unittest

import mock
from robot.errors import DataError, RobotError
from robot.result import model
from robot.utils.robottime import timestamp_to_secs

from xtesting.core import robotframework

__author__ = "Cedric Ollivier <cedric.ollivier@orange.com>"


class ResultVisitorTesting(unittest.TestCase):

    """The class testing ResultVisitor."""
    # pylint: disable=missing-docstring

    def setUp(self):
        self.visitor = robotframework.ResultVisitor()

    def test_empty(self):
        self.assertFalse(self.visitor.get_data())

    def test_ok(self):
        data = {'name': 'foo',
                'parent': 'bar',
                'status': 'PASS',
                'starttime': "20161216 16:00:00.000",
                'endtime': "20161216 16:00:01.000",
                'elapsedtime': 1000,
                'text': 'Hello, World!'}
        test = model.TestCase(
            name=data['name'], status=data['status'], message=data['text'],
            starttime=data['starttime'], endtime=data['endtime'])
        test.parent = mock.Mock()
        config = {'name': data['parent']}
        test.parent.configure_mock(**config)
        self.visitor.visit_test(test)
        self.assertEqual(self.visitor.get_data(), [data])


class ParseResultTesting(unittest.TestCase):

    """The class testing RobotFramework.parse_results()."""
    # pylint: disable=missing-docstring

    _config = {'name': 'dummy', 'starttime': '20161216 16:00:00.000',
               'endtime': '20161216 16:00:01.000'}

    def setUp(self):
        self.test = robotframework.RobotFramework(
            case_name='robot', project_name='xtesting')

    @mock.patch('robot.api.ExecutionResult', side_effect=DataError)
    def test_raises_exc(self, mock_method):
        with self.assertRaises(DataError):
            self.test.parse_results()
        mock_method.assert_called_once_with(
            os.path.join(self.test.res_dir, 'output.xml'))

    def _test_result(self, config, result):
        suite = mock.Mock()
        suite.configure_mock(**config)
        with mock.patch('robot.api.ExecutionResult',
                        return_value=mock.Mock(suite=suite)):
            self.test.parse_results()
            self.assertEqual(self.test.result, result)
            self.assertEqual(self.test.start_time,
                             timestamp_to_secs(config['starttime']))
            self.assertEqual(self.test.stop_time,
                             timestamp_to_secs(config['endtime']))
            self.assertEqual(self.test.details,
                             {'description': config['name'], 'tests': []})

    def test_null_passed(self):
        self._config.update({'statistics.passed': 0,
                             'statistics.total': 20})
        self._test_result(self._config, 0)

    def test_no_test(self):
        self._config.update({'statistics.passed': 20,
                             'statistics.total': 0})
        self._test_result(self._config, 0)

    def test_half_success(self):
        self._config.update({'statistics.passed': 10,
                             'statistics.total': 20})
        self._test_result(self._config, 50)

    def test_success(self):
        self._config.update({'statistics.passed': 20,
                             'statistics.total': 20})
        self._test_result(self._config, 100)


class GenerateReportTesting(unittest.TestCase):

    """The class testing RobotFramework.generate_report()."""
    # pylint: disable=missing-docstring

    def setUp(self):
        self.test = robotframework.RobotFramework(
            case_name='robot', project_name='xtesting')

    @mock.patch('robot.api.ExecutionResult', side_effect=Exception)
    def test_exc1(self, *args):
        with self.assertRaises(Exception):
            self.test.generate_report()
        args[0].assert_called_once_with(self.test.xml_file)

    @mock.patch('robot.reporting.resultwriter.ResultWriter.__init__',
                return_value=mock.Mock(side_effect=Exception))
    @mock.patch('robot.api.ExecutionResult')
    def test_exc2(self, *args):  # pylint: disable=unused-argument
        with self.assertRaises(Exception):
            self.test.generate_report()
        args[0].assert_called_once_with(self.test.xml_file)
        args[1].assert_called_once_with(args[0].return_value)

    @mock.patch('robot.reporting.resultwriter.ResultWriter.write_results',
                return_value=1)
    @mock.patch('robot.reporting.resultwriter.ResultWriter.__init__',
                return_value=None)
    @mock.patch('robot.api.ExecutionResult')
    def test_err(self, *args):
        self.assertEqual(self.test.generate_report(), 1)
        args[0].assert_called_once_with(self.test.xml_file)
        args[1].assert_called_once_with(args[0].return_value)
        args[2].assert_called_once_with(
            report=f'{self.test.res_dir}/report.html',
            log=f'{self.test.res_dir}/log.html',
            xunit=f'{self.test.res_dir}/xunit.xml')

    @mock.patch('robot.reporting.resultwriter.ResultWriter.write_results',
                return_value=0)
    @mock.patch('robot.reporting.resultwriter.ResultWriter.__init__',
                return_value=None)
    @mock.patch('robot.api.ExecutionResult')
    def test_ok(self, *args):  # pylint: disable=unused-argument
        self.assertEqual(self.test.generate_report(), 0)
        args[0].assert_called_once_with(self.test.xml_file)
        args[1].assert_called_once_with(args[0].return_value)
        args[2].assert_called_once_with(
            report=f'{self.test.res_dir}/report.html',
            log=f'{self.test.res_dir}/log.html',
            xunit=f'{self.test.res_dir}/xunit.xml')


class RunTesting(unittest.TestCase):

    """The class testing RobotFramework.run()."""
    # pylint: disable=missing-docstring

    suites = ["foo"]
    variable = []
    variablefile = []
    include = []

    def setUp(self):
        self.test = robotframework.RobotFramework(
            case_name='robot', project_name='xtesting')

    def test_exc_key_error(self):
        self.assertEqual(self.test.run(), self.test.EX_RUN_ERROR)

    @mock.patch('robot.run')
    def _test_makedirs_exc(self, *args):
        with mock.patch.object(self.test, 'parse_results') as mock_method, \
                mock.patch.object(self.test, 'generate_report') as mmethod:
            self.assertEqual(
                self.test.run(
                    suites=self.suites, variable=self.variable,
                    variablefile=self.variablefile, include=self.include),
                self.test.EX_RUN_ERROR)
            args[0].assert_not_called()
            mock_method.assert_not_called()
            mmethod.assert_not_called()

    @mock.patch('os.makedirs', side_effect=Exception)
    @mock.patch('os.path.exists', return_value=False)
    def test_makedirs_exc(self, *args):
        self._test_makedirs_exc()
        args[0].assert_called_once_with(self.test.res_dir)
        args[1].assert_called_once_with(self.test.res_dir)

    @mock.patch('robot.run')
    def _test_makedirs(self, *args):
        with mock.patch.object(self.test, 'parse_results') as mock_method, \
                mock.patch.object(self.test, 'generate_report',
                                  return_value=0) as mmethod:
            self.assertEqual(
                self.test.run(suites=self.suites, variable=self.variable),
                self.test.EX_OK)
            args[0].assert_called_once_with(
                *self.suites, log='NONE', output=self.test.xml_file,
                report='NONE', stdout=mock.ANY, variable=self.variable,
                variablefile=self.variablefile, include=self.include)
            mock_method.assert_called_once_with()
            mmethod.assert_called_once_with()

    @mock.patch('os.makedirs')
    @mock.patch('os.path.exists', return_value=True)
    def test_makedirs_oserror17(self, *args):
        self._test_makedirs()
        args[0].assert_called_once_with(self.test.res_dir)
        args[1].assert_not_called()

    @mock.patch('os.makedirs')
    @mock.patch('os.path.exists', return_value=False)
    def test_makedirs(self, *args):
        self._test_makedirs()
        args[0].assert_called_once_with(self.test.res_dir)
        args[1].assert_called_once_with(self.test.res_dir)

    @mock.patch('os.makedirs')
    @mock.patch('robot.run')
    def _test_parse_results(self, status, *args):
        self.assertEqual(
            self.test.run(
                suites=self.suites, variable=self.variable,
                variablefile=self.variablefile, include=self.include),
            status)
        args[0].assert_called_once_with(
            *self.suites, log='NONE', output=self.test.xml_file,
            report='NONE', stdout=mock.ANY, variable=self.variable,
            variablefile=self.variablefile, include=self.include)
        args[1].assert_called_once_with(self.test.res_dir)

    def test_parse_results_exc(self):
        with mock.patch.object(self.test, 'parse_results',
                               side_effect=Exception) as mock_method, \
                mock.patch.object(self.test, 'generate_report') as mmethod:
            self._test_parse_results(self.test.EX_RUN_ERROR)
            mock_method.assert_called_once_with()
            mmethod.assert_not_called()

    def test_parse_results_robot_error(self):
        with mock.patch.object(self.test, 'parse_results',
                               side_effect=RobotError('foo')) as mock_method, \
                mock.patch.object(self.test, 'generate_report') as mmethod:
            self._test_parse_results(self.test.EX_RUN_ERROR)
            mock_method.assert_called_once_with()
            mmethod.assert_not_called()

    @mock.patch('os.makedirs')
    @mock.patch('robot.run')
    def _test_generate_report(self, status, *args):
        with mock.patch.object(self.test, 'parse_results') as mock_method:
            self.assertEqual(
                self.test.run(
                    suites=self.suites, variable=self.variable,
                    variablefile=self.variablefile, include=self.include),
                status)
        args[0].assert_called_once_with(
            *self.suites, log='NONE', output=self.test.xml_file,
            report='NONE', stdout=mock.ANY, variable=self.variable,
            variablefile=self.variablefile, include=self.include)
        args[1].assert_called_once_with(self.test.res_dir)
        mock_method.assert_called_once_with()

    def test_generate_report_exc(self):
        with mock.patch.object(self.test, 'generate_report',
                               side_effect=Exception) as mmethod:
            self._test_generate_report(self.test.EX_RUN_ERROR)
            mmethod.assert_called_once_with()

    def test_generate_report_err(self):
        with mock.patch.object(self.test, 'generate_report',
                               return_value=1) as mmethod:
            self._test_generate_report(self.test.EX_OK)
            mmethod.assert_called_once_with()

    def test_generate_report(self):
        with mock.patch.object(self.test, 'generate_report',
                               return_value=0) as mmethod:
            self._test_generate_report(self.test.EX_OK)
            mmethod.assert_called_once_with()


if __name__ == "__main__":
    logging.disable(logging.CRITICAL)
    unittest.main(verbosity=2)



#!/usr/bin/env python

# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Apache License, Version 2.0
# which accompanies this distribution, and is available at
# http://www.apache.org/licenses/LICENSE-2.0

# pylint: disable=missing-docstring

import io
import logging
import subprocess
import unittest

import mock

from xtesting.core import unit
from xtesting.core import testcase


class SuiteTesting(unittest.TestCase):

    def setUp(self):
        self.psrunner = unit.Suite(case_name="unit")
        self.psrunner.suite = "foo"

    @mock.patch('subprocess.Popen', side_effect=Exception)
    def test_generate_stats_ko(self, *args):
        stream = io.StringIO()
        with self.assertRaises(Exception):
            self.psrunner.generate_stats(stream)
        args[0].assert_called_once_with(
            ['subunit-stats'], stdin=subprocess.PIPE, stdout=subprocess.PIPE)

    @mock.patch('subprocess.Popen')
    def test_generate_stats_ok(self, *args):
        attrs = {'return_value.__enter__.return_value': mock.Mock(
            communicate=mock.Mock(return_value=(b"foo", b"bar")))}
        args[0].configure_mock(**attrs)
        stream = io.StringIO()
        self.psrunner.generate_stats(stream)
        args[0].assert_called_once_with(
            ['subunit-stats'], stdin=subprocess.PIPE, stdout=subprocess.PIPE)

    @mock.patch('builtins.open', mock.mock_open())
    @mock.patch('subprocess.Popen', side_effect=Exception)
    def test_generate_xunit_ko(self, *args):
        stream = io.StringIO()
        with self.assertRaises(Exception), \
                mock.patch('builtins.open',
                           mock.mock_open()) as mock_open:
            self.psrunner.generate_xunit(stream)
        args[0].assert_called_once_with(
            ['subunit2junitxml'], stdin=subprocess.PIPE,
            stdout=subprocess.PIPE)
        mock_open.assert_called_once_with(
            f'{self.psrunner.res_dir}/results.xml', 'w', encoding='utf-8')

    @mock.patch('subprocess.Popen')
    def test_generate_xunit_ok(self, *args):
        attrs = {'return_value.__enter__.return_value': mock.Mock(
            communicate=mock.Mock(return_value=(b"foo", b"bar")))}
        args[0].configure_mock(**attrs)
        stream = io.BytesIO()
        with mock.patch('builtins.open',
                        mock.mock_open()) as mock_open:
            self.psrunner.generate_xunit(stream)
        args[0].assert_called_once_with(
            ['subunit2junitxml'], stdin=subprocess.PIPE,
            stdout=subprocess.PIPE)
        mock_open.assert_called_once_with(
            f'{self.psrunner.res_dir}/results.xml', 'w', encoding='utf-8')

    @mock.patch('subprocess.check_output', side_effect=Exception)
    def test_generate_html_ko(self, *args):
        stream = "foo"
        with self.assertRaises(Exception):
            self.psrunner.generate_html(stream)
        args[0].assert_called_once_with(
            ['subunit2html', stream,
             f'{self.psrunner.res_dir}/results.html'])

    @mock.patch('subprocess.check_output')
    def test_generate_html_ok(self, *args):
        stream = "foo"
        self.psrunner.generate_html(stream)
        args[0].assert_called_once_with(
            ['subunit2html', stream,
             f'{self.psrunner.res_dir}/results.html'])

    @mock.patch('xtesting.core.unit.Suite.generate_html')
    @mock.patch('xtesting.core.unit.Suite.generate_xunit')
    @mock.patch('xtesting.core.unit.Suite.generate_stats')
    @mock.patch('unittest.TestLoader')
    @mock.patch('subunit.run.SubunitTestRunner.run')
    def _test_run(self, mock_result, status, result, *args):
        args[0].return_value = mock_result
        with mock.patch('builtins.open', mock.mock_open()) as m_open:
            self.assertEqual(self.psrunner.run(), status)
        m_open.assert_called_once_with(
            f'{self.psrunner.res_dir}/subunit_stream', 'wb')
        self.assertEqual(self.psrunner.is_successful(), result)
        args[0].assert_called_once_with(self.psrunner.suite)
        args[1].assert_not_called()
        args[2].assert_called_once_with(mock.ANY)
        args[3].assert_called_once_with(mock.ANY)
        args[4].assert_called_once_with(
            f'{self.psrunner.res_dir}/subunit_stream')

    @mock.patch('xtesting.core.unit.Suite.generate_html')
    @mock.patch('xtesting.core.unit.Suite.generate_xunit')
    @mock.patch('xtesting.core.unit.Suite.generate_stats')
    @mock.patch('unittest.TestLoader')
    @mock.patch('subunit.run.SubunitTestRunner.run')
    def _test_run_name(self, name, mock_result, status, result, *args):
        args[0].return_value = mock_result
        with mock.patch('builtins.open', mock.mock_open()) as m_open:
            self.assertEqual(self.psrunner.run(name=name), status)
        m_open.assert_called_once_with(
            f'{self.psrunner.res_dir}/subunit_stream', 'wb')
        self.assertEqual(self.psrunner.is_successful(), result)
        args[0].assert_called_once_with(self.psrunner.suite)
        args[1].assert_called_once_with()
        args[2].assert_called_once_with(mock.ANY)
        args[3].assert_called_once_with(mock.ANY)
        args[4].assert_called_once_with(
            f'{self.psrunner.res_dir}/subunit_stream')

    @mock.patch('xtesting.core.unit.Suite.generate_html')
    @mock.patch('xtesting.core.unit.Suite.generate_xunit')
    @mock.patch('xtesting.core.unit.Suite.generate_stats')
    @mock.patch('unittest.TestLoader')
    @mock.patch('subunit.run.SubunitTestRunner.run')
    @mock.patch('os.path.isdir', return_value=True)
    def _test_run_exc(self, exc, *args):
        args[1].return_value = mock.Mock(
            decorated=mock.Mock(
                testsRun=50, errors=[], failures=[]))
        args[3].side_effect = exc
        with mock.patch('builtins.open',
                        mock.mock_open()) as m_open:
            self.assertEqual(
                self.psrunner.run(), testcase.TestCase.EX_RUN_ERROR)
        m_open.assert_not_called()
        self.assertEqual(
            self.psrunner.is_successful(),
            testcase.TestCase.EX_TESTCASE_FAILED)
        args[0].assert_called_once_with(self.psrunner.res_dir)
        args[1].assert_called_once_with(self.psrunner.suite)
        args[2].assert_not_called()
        args[3].assert_called_once_with(mock.ANY)
        args[4].assert_not_called()
        args[5].assert_not_called()

    def test_check_suite_null(self):
        self.assertEqual(unit.Suite().suite, None)
        self.psrunner.suite = None
        self.assertEqual(self.psrunner.run(), testcase.TestCase.EX_RUN_ERROR)

    @mock.patch('os.path.isdir', return_value=True)
    def test_run_no_ut(self, *args):
        mock_result = mock.Mock(
            decorated=mock.Mock(testsRun=0, errors=[], failures=[]))
        self._test_run(
            mock_result, testcase.TestCase.EX_RUN_ERROR,
            testcase.TestCase.EX_TESTCASE_FAILED)
        self.assertEqual(self.psrunner.result, 0)
        self.assertEqual(
            self.psrunner.details,
            {'errors': 0, 'failures': 0, 'testsRun': 0})
        args[0].assert_called_once_with(self.psrunner.res_dir)

    @mock.patch('os.path.isdir', return_value=True)
    def test_run_result_ko(self, *args):
        self.psrunner.criteria = 100
        mock_result = mock.Mock(
            decorated=mock.Mock(
                testsRun=50, errors=[('test1', 'error_msg1')],
                failures=[('test2', 'failure_msg1')]))
        self._test_run(
            mock_result, testcase.TestCase.EX_OK,
            testcase.TestCase.EX_TESTCASE_FAILED)
        self.assertEqual(self.psrunner.result, 96)
        self.assertEqual(
            self.psrunner.details,
            {'errors': 1, 'failures': 1, 'testsRun': 50})
        args[0].assert_called_once_with(self.psrunner.res_dir)

    @mock.patch('os.path.isdir', return_value=True)
    def test_run_result_ok_1(self, *args):
        mock_result = mock.Mock(
            decorated=mock.Mock(
                testsRun=50, errors=[], failures=[]))
        self._test_run(
            mock_result, testcase.TestCase.EX_OK,
            testcase.TestCase.EX_OK)
        self.assertEqual(self.psrunner.result, 100)
        self.assertEqual(
            self.psrunner.details,
            {'errors': 0, 'failures': 0, 'testsRun': 50})
        args[0].assert_called_once_with(self.psrunner.res_dir)

    @mock.patch('os.makedirs')
    @mock.patch('os.path.isdir', return_value=False)
    def test_run_result_ok_2(self, *args):
        mock_result = mock.Mock(
            decorated=mock.Mock(
                testsRun=50, errors=[], failures=[]))
        self._test_run(
            mock_result, testcase.TestCase.EX_OK,
            testcase.TestCase.EX_OK)
        self.assertEqual(self.psrunner.result, 100)
        self.assertEqual(
            self.psrunner.details,
            {'errors': 0, 'failures': 0, 'testsRun': 50})
        args[0].assert_called_once_with(self.psrunner.res_dir)
        args[1].assert_called_once_with(self.psrunner.res_dir)

    @mock.patch('unittest.TestLoader')
    def test_run_name_exc(self, mock_class=None):
        mock_obj = mock.Mock(side_effect=ImportError)
        mock_class.side_effect = mock_obj
        self.assertEqual(self.psrunner.run(name='foo'),
                         testcase.TestCase.EX_RUN_ERROR)
        mock_class.assert_called_once_with()
        mock_obj.assert_called_once_with()

    @mock.patch('os.path.isdir', return_value=True)
    def test_run_name(self, *args):
        mock_result = mock.Mock(
            decorated=mock.Mock(
                testsRun=50, errors=[], failures=[]))
        self._test_run_name(
            "foo", mock_result, testcase.TestCase.EX_OK,
            testcase.TestCase.EX_OK)
        self.assertEqual(self.psrunner.result, 100)
        self.assertEqual(
            self.psrunner.details,
            {'errors': 0, 'failures': 0, 'testsRun': 50})
        args[0].assert_called_once_with(self.psrunner.res_dir)

    def test_run_exc1(self):
        self._test_run_exc(AssertionError)

    def test_run_exc2(self):
        self._test_run_exc(ZeroDivisionError)

    def test_run_exc3(self):
        self._test_run_exc(Exception)


if __name__ == "__main__":
    logging.disable(logging.CRITICAL)
    unittest.main(verbosity=2)



#!/usr/bin/env python

# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Apache License, Version 2.0
# which accompanies this distribution, and is available at
# http://www.apache.org/licenses/LICENSE-2.0

# pylint: disable=missing-docstring

import logging
import unittest

import mock

from xtesting.ci import tier_handler


class TierHandlerTesting(unittest.TestCase):
    # pylint: disable=too-many-public-methods

    def setUp(self):
        self.test = mock.Mock()
        attrs = {'get_name.return_value': 'test_name'}
        self.test.configure_mock(**attrs)
        self.tier = tier_handler.Tier('test_tier', 'test_desc')
        self.testcase = tier_handler.TestCase(
            'test_name', 'true', False, 'test_criteria',
            True, description='test_desc', project='project_name')
        self.testcase.str = self.testcase.__str__()
        self.tier.str = self.tier.__str__()

    def test_add_test(self):
        self.tier.add_test(self.test)
        self.assertEqual(self.tier.tests_array, [self.test])

    def test_get_skipped_test1(self):
        self.assertEqual(self.tier.get_skipped_test(), [])

    def test_get_skipped_test2(self):
        self.tier.skip_test(self.test)
        self.assertEqual(self.tier.get_skipped_test(), [self.test])

    def test_get_tests(self):
        self.tier.tests_array = [self.test]
        self.assertEqual(self.tier.get_tests(), [self.test])

    def test_get_test_names(self):
        self.tier.tests_array = [self.test]
        self.assertEqual(self.tier.get_test_names(), ['test_name'])

    def test_get_test(self):
        self.tier.tests_array = [self.test]
        with mock.patch.object(self.tier, 'is_test', return_value=True):
            self.assertEqual(self.tier.get_test('test_name'), self.test)

    def test_get_test_missing_test(self):
        self.tier.tests_array = [self.test]
        with mock.patch.object(self.tier, 'is_test', return_value=False):
            self.assertEqual(self.tier.get_test('test_name'), None)

    def test_get_name(self):
        self.assertEqual(self.tier.get_name(), 'test_tier')

    def test_testcase_get_name(self):
        self.assertEqual(self.tier.get_name(), 'test_tier')

    def test_testcase_is_enabled(self):
        self.assertEqual(self.testcase.is_enabled(), 'true')

    def test_testcase_get_criteria(self):
        self.assertEqual(self.testcase.get_criteria(), 'test_criteria')

    def test_testcase_is_blocking(self):
        self.assertTrue(self.testcase.is_blocking())

    def test_testcase_get_project(self):
        self.assertEqual(self.testcase.get_project(), 'project_name')


if __name__ == "__main__":
    logging.disable(logging.CRITICAL)
    unittest.main(verbosity=2)



#!/usr/bin/env python

# Copyright (c) 2016 Ericsson AB and others.
#
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Apache License, Version 2.0
# which accompanies this distribution, and is available at
# http://www.apache.org/licenses/LICENSE-2.0

""" The entry of running tests:
1) Parses xtesting/ci/testcases.yaml to check which testcase(s) to be run
2) Execute the common operations on every testcase (run, push results to db...)
3) Return the right status code
"""

import argparse
import errno
import logging
import logging.config
import os
import re
import sys
import textwrap

import enum
import pkg_resources
import prettytable
from stevedore import driver
import yaml

from xtesting.ci import tier_builder
from xtesting.core import testcase
from xtesting.utils import constants
from xtesting.utils import env

LOGGER = logging.getLogger('xtesting.ci.run_tests')


class Result(enum.Enum):
    """The overall result in enumerated type"""
    # pylint: disable=too-few-public-methods
    EX_OK = os.EX_OK
    EX_ERROR = -1


class BlockingTestFailed(Exception):
    """Exception when the blocking test fails"""


class RunTestsParser():
    """Parser to run tests"""
    # pylint: disable=too-few-public-methods

    def __init__(self):
        self.parser = argparse.ArgumentParser()
        self.parser.add_argument("-t", "--test", dest="test", action='store',
                                 help="Test case or tier (group of tests) "
                                 "to be executed. It will run all the test "
                                 "if not specified.")
        self.parser.add_argument("-n", "--noclean", help="Do not clean "
                                 "OpenStack resources after running each "
                                 "test (default=false).",
                                 action="store_true")
        self.parser.add_argument("-r", "--report", help="Push results to "
                                 "database (default=false).",
                                 action="store_true")
        self.parser.add_argument("-p", "--push", help="Push artifacts to "
                                 "S3 repository (default=false).",
                                 action="store_true")

    def parse_args(self, argv=None):
        """Parse arguments.

        It can call sys.exit if arguments are incorrect.

        Returns:
            the arguments from cmdline
        """
        return vars(self.parser.parse_args(argv))


class Runner():
    """Runner class"""

    def __init__(self):
        self.executed_test_cases = {}
        self.overall_result = Result.EX_OK
        self.clean_flag = True
        self.report_flag = False
        self.push_flag = False
        self.tiers = tier_builder.TierBuilder(
            pkg_resources.resource_filename('xtesting', 'ci/testcases.yaml'))

    @staticmethod
    def source_envfile(rc_file=constants.ENV_FILE):
        """Source the env file passed as arg"""
        if not os.path.isfile(rc_file):
            LOGGER.debug("No env file %s found", rc_file)
            return
        with open(rc_file, "r", encoding='utf-8') as rcfd:
            for line in rcfd:
                var = (line.rstrip('"\n').replace('export ', '').split(
                    "=") if re.search(r'(.*)=(.*)', line) else None)
                # The two next lines should be modified as soon as rc_file
                # conforms with common rules. Be aware that it could induce
                # issues if value starts with '
                if var:
                    key = re.sub(r'^["\' ]*|[ \'"]*$', '', var[0])
                    value = re.sub(r'^["\' ]*|[ \'"]*$', '', "".join(var[1:]))
                    os.environ[key] = value
            rcfd.seek(0, 0)
            LOGGER.debug("Sourcing env file %s\n\n%s", rc_file, rcfd.read())

    @staticmethod
    def get_dict_by_test(testname):
        # pylint: disable=missing-docstring
        with open(pkg_resources.resource_filename(
                'xtesting', 'ci/testcases.yaml'), encoding='utf-8') as tyaml:
            testcases_yaml = yaml.safe_load(tyaml)
        for dic_tier in testcases_yaml.get("tiers"):
            for dic_testcase in dic_tier['testcases']:
                if dic_testcase['case_name'] == testname:
                    return dic_testcase
        LOGGER.error('Project %s is not defined in testcases.yaml', testname)
        return None

    @staticmethod
    def get_run_dict(testname):
        """Obtain the 'run' block of the testcase from testcases.yaml"""
        try:
            dic_testcase = Runner.get_dict_by_test(testname)
            if not dic_testcase:
                LOGGER.error("Cannot get %s's config options", testname)
            elif 'run' in dic_testcase:
                return dic_testcase['run']
            return None
        except Exception:  # pylint: disable=broad-except
            LOGGER.exception("Cannot get %s's config options", testname)
            return None

    def run_test(self, test):
        """Run one test case"""
        if not test.is_enabled() or test.is_skipped():
            msg = prettytable.PrettyTable(
                header_style='upper', padding_width=5,
                field_names=['test case', 'project', 'duration',
                             'result'])
            msg.add_row([test.get_name(), test.get_project(), "00:00", "SKIP"])
            LOGGER.info("Test result:\n\n%s\n", msg)
            return testcase.TestCase.EX_TESTCASE_SKIPPED
        result = testcase.TestCase.EX_TESTCASE_FAILED
        run_dict = self.get_run_dict(test.get_name())
        if run_dict:
            try:
                LOGGER.info("Loading test case '%s'...", test.get_name())
                test_dict = Runner.get_dict_by_test(test.get_name())
                test_case = driver.DriverManager(
                    namespace='xtesting.testcase',
                    name=run_dict['name'],
                    invoke_on_load=True,
                    invoke_kwds=test_dict).driver
                self.executed_test_cases[test.get_name()] = test_case
                test_case.check_requirements()
                if test_case.is_skipped:
                    LOGGER.info("Skipping test case '%s'...", test.get_name())
                    LOGGER.info("Test result:\n\n%s\n", test_case)
                    return testcase.TestCase.EX_TESTCASE_SKIPPED
                LOGGER.info("Running test case '%s'...", test.get_name())
                try:
                    kwargs = run_dict['args']
                    test_case.run(**kwargs)
                except KeyError:
                    test_case.run()
                result = test_case.is_successful()
                LOGGER.info("Test result:\n\n%s\n", test_case)
                if self.clean_flag:
                    test_case.clean()
                if self.push_flag:
                    test_case.publish_artifacts()
                if self.report_flag:
                    test_case.push_to_db()
            except ImportError:
                LOGGER.exception("Cannot import module %s", run_dict['module'])
            except AttributeError:
                LOGGER.exception("Cannot get class %s", run_dict['class'])
            except Exception:  # pylint: disable=broad-except
                LOGGER.exception(
                    "\n\nPlease fix the testcase %s.\n"
                    "All exceptions should be caught by the testcase instead!"
                    "\n\n",
                    test.get_name())
        else:
            raise Exception("Cannot import the class for the test case.")
        return result

    def run_tier(self, tier):
        """Run one tier"""
        tests = tier.get_tests()
        if not tests:
            LOGGER.info("There are no supported test cases in this tier "
                        "for the given scenario")
            self.overall_result = Result.EX_ERROR
        else:
            for test in tests:
                self.run_test(test)
                test_case = self.executed_test_cases[test.get_name()]
                if test_case.is_successful() == test_case.EX_TESTCASE_FAILED:
                    LOGGER.error("The test case '%s' failed.", test.get_name())
                    self.overall_result = Result.EX_ERROR
                    if test.is_blocking():
                        raise BlockingTestFailed(
                            f"The test case {test.get_name()} "
                            "failed and is blocking")
        return self.overall_result

    def run_all(self):
        """Run all available testcases"""
        tiers_to_run = []
        msg = prettytable.PrettyTable(
            header_style='upper', padding_width=5,
            field_names=['tiers', 'description', 'testcases'])
        for tier in self.tiers.get_tiers():
            if tier.get_tests():
                tiers_to_run.append(tier)
                msg.add_row([tier.get_name(),
                             textwrap.fill(tier.description, width=40),
                             textwrap.fill(' '.join([str(x.get_name(
                                 )) for x in tier.get_tests()]), width=40)])
        LOGGER.info("TESTS TO BE EXECUTED:\n\n%s\n", msg)
        for tier in tiers_to_run:
            self.run_tier(tier)

    def main(self, **kwargs):  # pylint: disable=too-many-branches
        """Entry point of class Runner"""
        if 'noclean' in kwargs:
            self.clean_flag = not kwargs['noclean']
        if 'report' in kwargs:
            self.report_flag = kwargs['report']
        if 'push' in kwargs:
            self.push_flag = kwargs['push']
        try:
            LOGGER.info("Deployment description:\n\n%s\n", env.string())
            self.source_envfile()
            if 'test' in kwargs:
                LOGGER.debug("Test args: %s", kwargs['test'])
                if self.tiers.get_tier(kwargs['test']):
                    self.run_tier(self.tiers.get_tier(kwargs['test']))
                elif self.tiers.get_test(kwargs['test']):
                    result = self.run_test(
                        self.tiers.get_test(kwargs['test']))
                    if result == testcase.TestCase.EX_TESTCASE_FAILED:
                        LOGGER.error("The test case '%s' failed.",
                                     kwargs['test'])
                        self.overall_result = Result.EX_ERROR
                elif kwargs['test'] == "all":
                    self.run_all()
                else:
                    LOGGER.error("Unknown test case or tier '%s', or not "
                                 "supported by the given scenario '%s'.",
                                 kwargs['test'],
                                 env.get('DEPLOY_SCENARIO'))
                    LOGGER.debug("Available tiers are:\n\n%s",
                                 self.tiers)
                    return Result.EX_ERROR
            else:
                self.run_all()
        except BlockingTestFailed:
            pass
        except Exception:  # pylint: disable=broad-except
            LOGGER.exception("Failures when running testcase(s)")
            self.overall_result = Result.EX_ERROR
        if not self.tiers.get_test(kwargs['test']):
            self.summary(self.tiers.get_tier(kwargs['test']))
        LOGGER.info("Execution exit value: %s", self.overall_result)
        return self.overall_result

    def summary(self, tier=None):
        """To generate xtesting report showing the overall results"""
        msg = prettytable.PrettyTable(
            header_style='upper', padding_width=5,
            field_names=['test case', 'project', 'tier',
                         'duration', 'result'])
        tiers = [tier] if tier else self.tiers.get_tiers()
        for each_tier in tiers:
            for test in each_tier.get_tests():
                try:
                    test_case = self.executed_test_cases[test.get_name()]
                except KeyError:
                    msg.add_row([test.get_name(), test.get_project(),
                                 each_tier.get_name(), "00:00", "SKIP"])
                else:
                    if test_case.is_skipped:
                        result = 'SKIP'
                    else:
                        result = 'PASS' if(test_case.is_successful(
                            ) == test_case.EX_OK) else 'FAIL'
                    msg.add_row(
                        [test_case.case_name, test_case.project_name,
                         self.tiers.get_tier_name(test_case.case_name),
                         test_case.get_duration(), result])
            for test in each_tier.get_skipped_test():
                msg.add_row([test.get_name(), test.get_project(),
                             each_tier.get_name(), "00:00", "SKIP"])
        LOGGER.info("Xtesting report:\n\n%s\n", msg)


def main():
    """Entry point"""
    try:
        os.makedirs(constants.RESULTS_DIR)
    except OSError as ex:
        if ex.errno != errno.EEXIST:
            print(f"Cannot create {constants.RESULTS_DIR}")
            return testcase.TestCase.EX_RUN_ERROR
    if env.get('DEBUG').lower() == 'true':
        logging.config.fileConfig(pkg_resources.resource_filename(
            'xtesting', constants.DEBUG_INI_PATH))
    else:
        logging.config.fileConfig(pkg_resources.resource_filename(
            'xtesting', constants.INI_PATH))
    logging.captureWarnings(True)
    parser = RunTestsParser()
    args = parser.parse_args(sys.argv[1:])
    runner = Runner()
    return runner.main(**args).value



#!/usr/bin/env python

# Copyright (c) 2017 Orange and others.
#
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Apache License, Version 2.0
# which accompanies this distribution, and is available at
# http://www.apache.org/licenses/LICENSE-2.0

# pylint: disable=missing-docstring

import os

from io import BytesIO
import logging
import subprocess
import unittest

import mock

from xtesting.core import feature
from xtesting.core import testcase
from xtesting.utils import constants


class FakeTestCase(feature.Feature):

    def execute(self, **kwargs):
        pass


class AbstractFeatureTesting(unittest.TestCase):

    def test_run_unimplemented(self):
        # pylint: disable=abstract-class-instantiated
        with self.assertRaises(TypeError):
            feature.Feature(case_name="feature", project_name="xtesting")


class FeatureTestingBase(unittest.TestCase):

    _case_name = "foo"
    _project_name = "bar"
    _repo = "dir_repo_bar"
    _cmd = "run_bar_tests.py"
    _output_file = os.path.join(constants.RESULTS_DIR, 'foo/foo.log')
    _max_duration = 1
    feature = None

    @mock.patch('time.time', side_effect=[1, 2])
    def _test_run(self, status, mock_method=None):
        self.assertEqual(self.feature.run(cmd=self._cmd), status)
        if status == testcase.TestCase.EX_OK:
            self.assertEqual(self.feature.result, 100)
        else:
            self.assertEqual(self.feature.result, 0)
        mock_method.assert_has_calls([mock.call(), mock.call()])
        self.assertEqual(self.feature.start_time, 1)
        self.assertEqual(self.feature.stop_time, 2)

    @mock.patch('time.time', side_effect=[1, 2])
    def _test_run_console(self, console, status, mock_method=None):
        self.assertEqual(
            self.feature.run(cmd=self._cmd, console=console), status)
        self.assertEqual(self.feature.result, 100)
        mock_method.assert_has_calls([mock.call(), mock.call()])
        self.assertEqual(self.feature.start_time, 1)
        self.assertEqual(self.feature.stop_time, 2)

    @mock.patch('time.time', side_effect=[1, 2])
    def _test_run_max_duration(self, status, mock_method=None):
        self.assertEqual(
            self.feature.run(cmd=self._cmd, max_duration=self._max_duration),
            status)
        mock_method.assert_has_calls([mock.call(), mock.call()])
        self.assertEqual(self.feature.start_time, 1)
        self.assertEqual(self.feature.stop_time, 2)


class FeatureTesting(FeatureTestingBase):

    def setUp(self):
        # logging must be disabled else it calls time.time()
        # what will break these unit tests.
        logging.disable(logging.CRITICAL)
        with mock.patch('builtins.open'):
            self.feature = FakeTestCase(
                project_name=self._project_name, case_name=self._case_name)

    def test_run_exc(self):
        with mock.patch.object(
                self.feature, 'execute',
                side_effect=Exception) as mock_method:
            self._test_run(testcase.TestCase.EX_RUN_ERROR)
            mock_method.assert_called_once_with(cmd=self._cmd)

    def test_run(self):
        self._test_run(testcase.TestCase.EX_RUN_ERROR)


class BashFeatureTesting(FeatureTestingBase):

    def setUp(self):
        # logging must be disabled else it calls time.time()
        # what will break these unit tests.
        logging.disable(logging.CRITICAL)
        with mock.patch('builtins.open'):
            self.feature = feature.BashFeature(
                project_name=self._project_name, case_name=self._case_name)

    @mock.patch('subprocess.Popen')
    def test_run_no_cmd(self, mock_subproc):
        self.assertEqual(
            self.feature.run(), testcase.TestCase.EX_RUN_ERROR)
        mock_subproc.assert_not_called()

    @mock.patch('os.path.isdir', return_value=True)
    @mock.patch('subprocess.Popen',
                side_effect=subprocess.CalledProcessError(0, '', ''))
    def test_run_ko1(self, *args):
        with mock.patch('builtins.open', mock.mock_open()) as mopen:
            self._test_run(testcase.TestCase.EX_RUN_ERROR)
        mopen.assert_called_once_with(self._output_file, "w", encoding='utf-8')
        args[0].assert_called_once_with(
            self._cmd, shell=True, stderr=mock.ANY, stdout=mock.ANY)
        args[1].assert_called_once_with(self.feature.res_dir)

    @mock.patch('os.path.isdir', return_value=True)
    @mock.patch('subprocess.Popen')
    def test_run_ko2(self, *args):
        stream = BytesIO()
        stream.write(b"foo")
        stream.seek(0)
        attrs = {'return_value.__enter__.return_value.stdout': stream,
                 'return_value.__enter__.return_value.returncode': 1}
        args[0].configure_mock(**attrs)
        with mock.patch('builtins.open', mock.mock_open()) as mopen:
            self._test_run(testcase.TestCase.EX_RUN_ERROR)
        self.assertIn(
            mock.call(self._output_file, 'w', encoding='utf-8'),
            mopen.mock_calls)
        self.assertIn(
            mock.call(self._output_file, 'r', encoding='utf-8'),
            mopen.mock_calls)
        args[0].assert_called_once_with(
            self._cmd, shell=True, stderr=mock.ANY, stdout=mock.ANY)
        args[1].assert_called_once_with(self.feature.res_dir)

    @mock.patch('subprocess.Popen')
    @mock.patch('os.path.isdir', return_value=True)
    def test_run_ko3(self, *args):
        stream = BytesIO()
        stream.write(b"foo")
        stream.seek(0)
        wait = mock.MagicMock(side_effect=subprocess.TimeoutExpired(
            cmd=FeatureTestingBase._cmd,
            timeout=FeatureTestingBase._max_duration))
        kill = mock.MagicMock()
        attrs = {'return_value.__enter__.return_value.wait': wait,
                 'return_value.__enter__.return_value.kill': kill,
                 'return_value.__enter__.return_value.stdout': stream,
                 'return_value.__enter__.return_value.returncode': 0}
        args[1].configure_mock(**attrs)
        with mock.patch('builtins.open', mock.mock_open()) as mopen:
            self._test_run_max_duration(testcase.TestCase.EX_RUN_ERROR)
        self.assertIn(
            mock.call(self._output_file, 'w', encoding='utf-8'),
            mopen.mock_calls)
        self.assertNotIn(
            mock.call(self._output_file, 'r', encoding='utf-8'),
            mopen.mock_calls)
        args[1].assert_called_once_with(
            self._cmd, shell=True, stderr=mock.ANY, stdout=mock.ANY)
        wait.assert_called_once_with(timeout=FeatureTestingBase._max_duration)
        kill.assert_called_once()
        args[0].assert_called_once_with(self.feature.res_dir)

    @mock.patch('os.path.isdir', return_value=True)
    @mock.patch('subprocess.Popen')
    def test_run1(self, *args):
        stream = BytesIO()
        stream.write(b"foo")
        stream.seek(0)
        attrs = {'return_value.__enter__.return_value.stdout': stream,
                 'return_value.__enter__.return_value.returncode': 0}
        args[0].configure_mock(**attrs)
        with mock.patch('builtins.open', mock.mock_open()) as mopen:
            self._test_run(testcase.TestCase.EX_OK)
        self.assertIn(
            mock.call(self._output_file, 'w', encoding='utf-8'),
            mopen.mock_calls)
        self.assertIn(
            mock.call(self._output_file, 'r', encoding='utf-8'),
            mopen.mock_calls)
        args[0].assert_called_once_with(
            self._cmd, shell=True, stderr=mock.ANY, stdout=mock.ANY)
        args[1].assert_called_once_with(self.feature.res_dir)

    @mock.patch('os.path.isdir', return_value=True)
    @mock.patch('subprocess.Popen')
    def test_run2(self, *args):
        stream = BytesIO()
        stream.write(b"foo")
        stream.seek(0)
        attrs = {'return_value.__enter__.return_value.stdout': stream,
                 'return_value.__enter__.return_value.returncode': 0}
        args[0].configure_mock(**attrs)
        with mock.patch('builtins.open', mock.mock_open()) as mopen:
            self._test_run_console(True, testcase.TestCase.EX_OK)
        self.assertIn(
            mock.call(self._output_file, 'w', encoding='utf-8'),
            mopen.mock_calls)
        self.assertIn(
            mock.call(self._output_file, 'r', encoding='utf-8'),
            mopen.mock_calls)
        args[0].assert_called_once_with(
            self._cmd, shell=True, stderr=mock.ANY, stdout=mock.ANY)
        args[1].assert_called_once_with(self.feature.res_dir)

    @mock.patch('os.path.isdir', return_value=True)
    @mock.patch('subprocess.Popen')
    def test_run3(self, *args):
        stream = BytesIO()
        stream.write(b"foo")
        stream.seek(0)
        attrs = {'return_value.__enter__.return_value.stdout': stream,
                 'return_value.__enter__.return_value.returncode': 0}
        args[0].configure_mock(**attrs)
        with mock.patch('builtins.open', mock.mock_open()) as mopen:
            self._test_run_console(False, testcase.TestCase.EX_OK)
        self.assertIn(
            mock.call(self._output_file, 'w', encoding='utf-8'),
            mopen.mock_calls)
        self.assertIn(
            mock.call(self._output_file, 'r', encoding='utf-8'),
            mopen.mock_calls)
        args[0].assert_called_once_with(
            self._cmd, shell=True, stderr=mock.ANY, stdout=mock.ANY)
        args[1].assert_called_once_with(self.feature.res_dir)

    @mock.patch('os.makedirs')
    @mock.patch('os.path.isdir', return_value=False)
    @mock.patch('subprocess.Popen')
    def test_run4(self, *args):
        stream = BytesIO()
        stream.write(b"foo")
        stream.seek(0)
        attrs = {'return_value.__enter__.return_value.stdout': stream,
                 'return_value.__enter__.return_value.returncode': 0}
        args[0].configure_mock(**attrs)
        with mock.patch('builtins.open', mock.mock_open()) as mopen:
            self._test_run_console(False, testcase.TestCase.EX_OK)
        self.assertIn(
            mock.call(self._output_file, 'w', encoding='utf-8'),
            mopen.mock_calls)
        self.assertIn(
            mock.call(self._output_file, 'r', encoding='utf-8'),
            mopen.mock_calls)
        args[0].assert_called_once_with(
            self._cmd, shell=True, stderr=mock.ANY, stdout=mock.ANY)
        args[1].assert_called_once_with(self.feature.res_dir)
        args[2].assert_called_once_with(self.feature.res_dir)


if __name__ == "__main__":
    unittest.main(verbosity=2)



#!/usr/bin/env python

# Copyright (c) 2016 Ericsson AB and others.
#
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Apache License, Version 2.0
# which accompanies this distribution, and is available at
# http://www.apache.org/licenses/LICENSE-2.0

# pylint: disable=missing-docstring,too-many-instance-attributes

"""Tier and TestCase classes to wrap the testcases config file"""

import textwrap

import prettytable


class Tier():

    def __init__(self, name, description=""):
        self.tests_array = []
        self.skipped_tests_array = []
        self.name = name
        self.description = description

    def add_test(self, testcase):
        self.tests_array.append(testcase)

    def skip_test(self, testcase):
        self.skipped_tests_array.append(testcase)

    def get_tests(self):
        array_tests = []
        for test in self.tests_array:
            array_tests.append(test)
        return array_tests

    def get_skipped_test(self):
        return self.skipped_tests_array

    def get_test_names(self):
        array_tests = []
        for test in self.tests_array:
            array_tests.append(test.get_name())
        return array_tests

    def get_test(self, test_name):
        if self.is_test(test_name):
            for test in self.tests_array + self.skipped_tests_array:
                if test.get_name() == test_name:
                    return test
        return None

    def is_test(self, test_name):
        for test in self.tests_array + self.skipped_tests_array:
            if test.get_name() == test_name:
                return True
        return False

    def get_name(self):
        return self.name

    def __str__(self):
        msg = prettytable.PrettyTable(
            header_style='upper', padding_width=5,
            field_names=['tiers', 'description',
                         'testcases'])
        msg.add_row(
            [self.name, textwrap.fill(self.description, width=40),
             textwrap.fill(' '.join([str(x.get_name(
                 )) for x in self.get_tests()]), width=40)])
        return msg.get_string()


class TestCase():

    def __init__(self, name, enabled, skipped, criteria, blocking,
                 description="", project=""):
        # pylint: disable=too-many-arguments
        self.name = name
        self.enabled = enabled
        self.skipped = skipped
        self.criteria = criteria
        self.blocking = blocking
        self.description = description
        self.project = project

    def get_name(self):
        return self.name

    def is_enabled(self):
        return self.enabled

    def is_skipped(self):
        return self.skipped

    def get_criteria(self):
        return self.criteria

    def is_blocking(self):
        return self.blocking

    def get_project(self):
        return self.project

    def __str__(self):
        msg = prettytable.PrettyTable(
            header_style='upper', padding_width=5,
            field_names=['test case', 'description', 'criteria'])
        msg.add_row([self.name, textwrap.fill(self.description, width=40),
                     self.criteria])
        return msg.get_string()



#!/usr/bin/env python

# Copyright (c) 2016 ZTE Corp and others.
#
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Apache License, Version 2.0
# which accompanies this distribution, and is available at
# http://www.apache.org/licenses/LICENSE-2.0

"""Define the parent classes of all Xtesting Features.

Feature is considered as TestCase offered by Third-party. It offers
helpers to run any python method or any bash command.
"""

import abc
import logging
import os
import subprocess
import sys
import time

from xtesting.core import testcase

__author__ = ("Serena Feng <feng.xiaowei@zte.com.cn>, "
              "Cedric Ollivier <cedric.ollivier@orange.com>")


class Feature(testcase.TestCase, metaclass=abc.ABCMeta):
    """Base model for single feature."""

    __logger = logging.getLogger(__name__)

    @abc.abstractmethod
    def execute(self, **kwargs):
        """Execute the Python method.

        The subclasses must override the default implementation which
        is false on purpose.

        The new implementation must return 0 if success or anything
        else if failure.

        Args:
            kwargs: Arbitrary keyword arguments.
        """

    def run(self, **kwargs):
        """Run the feature.

        It allows executing any Python method by calling execute().

        It sets the following attributes required to push the results
        to DB:

            * result,
            * start_time,
            * stop_time.

        It doesn't fulfill details when pushing the results to the DB.

        Args:
            kwargs: Arbitrary keyword arguments.

        Returns:
            TestCase.EX_OK if execute() returns 0,
            TestCase.EX_RUN_ERROR otherwise.
        """
        self.start_time = time.time()
        exit_code = testcase.TestCase.EX_RUN_ERROR
        self.result = 0
        try:
            if self.execute(**kwargs) == 0:
                exit_code = testcase.TestCase.EX_OK
                self.result = 100
        except Exception:  # pylint: disable=broad-except
            self.__logger.exception("%s FAILED", self.project_name)
        self.stop_time = time.time()
        return exit_code


class BashFeature(Feature):
    """Class designed to run any bash command."""

    __logger = logging.getLogger(__name__)

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.result_file = f"{self.res_dir}/{self.case_name}.log"

    def execute(self, **kwargs):
        """Execute the cmd passed as arg

        Args:
            kwargs: Arbitrary keyword arguments.

        Returns:
            process return code if no exception,
            -1 otherwise.
        """
        try:
            cmd = kwargs["cmd"]
            console = kwargs["console"] if "console" in kwargs else False
            # For some tests, we may need to force stop after N sec
            max_duration = kwargs.get("max_duration")
            if not os.path.isdir(self.res_dir):
                os.makedirs(self.res_dir)
            with open(self.result_file, 'w', encoding='utf-8') as f_stdout:
                self.__logger.info("Calling %s", cmd)
                with subprocess.Popen(
                        cmd, shell=True, stdout=subprocess.PIPE,
                        stderr=subprocess.STDOUT) as process:
                    for line in iter(process.stdout.readline, b''):
                        if console:
                            sys.stdout.write(line.decode("utf-8"))
                        f_stdout.write(line.decode("utf-8"))
                    try:
                        process.wait(timeout=max_duration)
                    except subprocess.TimeoutExpired:
                        process.kill()
                        self.__logger.info(
                            "Killing process after %d second(s).",
                            max_duration)
                        return -2
            with open(self.result_file, 'r', encoding='utf-8') as f_stdin:
                self.__logger.debug("$ %s\n%s", cmd, f_stdin.read().rstrip())
            return process.returncode
        except KeyError:
            self.__logger.error("Please give cmd as arg. kwargs: %s", kwargs)
        return -1



#!/usr/bin/env python

# Copyright (c) 2016 Orange and others.
#
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Apache License, Version 2.0
# which accompanies this distribution, and is available at
# http://www.apache.org/licenses/LICENSE-2.0

"""Define the parent class of all Xtesting TestCases."""

import abc
from datetime import datetime
import json
import logging
import mimetypes
import os
import re

from urllib.parse import urlparse
import boto3
from boto3.s3.transfer import TransferConfig
import botocore
import prettytable
import requests

from xtesting.utils import decorators
from xtesting.utils import env
from xtesting.utils import constants

__author__ = "Cedric Ollivier <cedric.ollivier@orange.com>"


class TestCase(metaclass=abc.ABCMeta):
    # pylint: disable=too-many-instance-attributes
    """Base model for single test case."""

    EX_OK = os.EX_OK
    """everything is OK"""

    EX_RUN_ERROR = os.EX_SOFTWARE
    """run() failed"""

    EX_PUSH_TO_DB_ERROR = os.EX_SOFTWARE - 1
    """push_to_db() failed"""

    EX_TESTCASE_FAILED = os.EX_SOFTWARE - 2
    """results are false"""

    EX_TESTCASE_SKIPPED = os.EX_SOFTWARE - 3
    """requirements are unmet"""

    EX_PUBLISH_ARTIFACTS_ERROR = os.EX_SOFTWARE - 4
    """publish_artifacts() failed"""

    dir_results = constants.RESULTS_DIR
    _job_name_rule = "(dai|week)ly-(.+?)-[0-9]*"
    headers = {'Content-Type': 'application/json'}
    __logger = logging.getLogger(__name__)

    def __init__(self, **kwargs):
        self.details = {}
        self.project_name = kwargs.get('project_name', 'xtesting')
        self.case_name = kwargs.get('case_name', '')
        self.criteria = kwargs.get('criteria', 100)
        self.result = 0
        self.start_time = 0
        self.stop_time = 0
        self.is_skipped = False
        self.output_log_name = os.path.basename(constants.LOG_PATH)
        self.output_debug_log_name = os.path.basename(constants.DEBUG_LOG_PATH)
        self.res_dir = os.path.join(self.dir_results, self.case_name)

    def __str__(self):
        try:
            assert self.project_name
            assert self.case_name
            if self.is_skipped:
                result = 'SKIP'
            else:
                result = 'PASS' if(self.is_successful(
                    ) == TestCase.EX_OK) else 'FAIL'
            msg = prettytable.PrettyTable(
                header_style='upper', padding_width=5,
                field_names=['test case', 'project', 'duration',
                             'result'])
            msg.add_row([self.case_name, self.project_name,
                         self.get_duration(), result])
            return msg.get_string()
        except AssertionError:
            self.__logger.error("We cannot print invalid objects")
            return super().__str__()

    def get_duration(self):
        """Return the duration of the test case.

        Returns:
            duration if start_time and stop_time are set
            "XX:XX" otherwise.
        """
        try:
            if self.is_skipped:
                return "00:00"
            assert self.start_time
            assert self.stop_time
            if self.stop_time < self.start_time:
                return "XX:XX"
            return(
                f"{str(int(self.stop_time - self.start_time) // 60).zfill(2)}:"
                f"{str(int(self.stop_time - self.start_time) % 60).zfill(2)}")

        except Exception:  # pylint: disable=broad-except
            self.__logger.error("Please run test before getting the duration")
            return "XX:XX"

    def is_successful(self):
        """Interpret the result of the test case.

        It allows getting the result of TestCase. It completes run()
        which only returns the execution status.

        It can be overriden if checking result is not suitable.

        Returns:
            TestCase.EX_OK if result is 'PASS'.
            TestCase.EX_TESTCASE_SKIPPED if test case is skipped.
            TestCase.EX_TESTCASE_FAILED otherwise.
        """
        try:
            if self.is_skipped:
                return TestCase.EX_TESTCASE_SKIPPED
            assert self.criteria
            assert self.result is not None
            if (not isinstance(self.result, str) and
                    not isinstance(self.criteria, str)):
                if self.result >= self.criteria:
                    return TestCase.EX_OK
            else:
                # Backward compatibility
                # It must be removed as soon as TestCase subclasses
                # stop setting result = 'PASS' or 'FAIL'.
                # In this case criteria is unread.
                self.__logger.warning(
                    "Please update result which must be an int!")
                if self.result == 'PASS':
                    return TestCase.EX_OK
        except AssertionError:
            self.__logger.error("Please run test before checking the results")
        return TestCase.EX_TESTCASE_FAILED

    def check_requirements(self):  # pylint: disable=no-self-use
        """Check the requirements of the test case.

        It can be overriden on purpose.
        """
        self.is_skipped = False

    @abc.abstractmethod
    def run(self, **kwargs):
        """Run the test case.

        It allows running TestCase and getting its execution
        status.

        The subclasses must override the default implementation which
        is false on purpose.

        The new implementation must set the following attributes to
        push the results to DB:

            * result,
            * start_time,
            * stop_time.

        Args:
            kwargs: Arbitrary keyword arguments.
        """

    @decorators.can_dump_request_to_file
    def push_to_db(self):
        """Push the results of the test case to the DB.

        It allows publishing the results and checking the status.

        It could be overriden if the common implementation is not
        suitable.

        The following attributes must be set before pushing the results to DB:

            * project_name,
            * case_name,
            * result,
            * start_time,
            * stop_time.

        The next vars must be set in env:

            * TEST_DB_URL,
            * INSTALLER_TYPE,
            * DEPLOY_SCENARIO,
            * NODE_NAME,
            * BUILD_TAG.

        Returns:
            TestCase.EX_OK if results were pushed to DB.
            TestCase.EX_PUSH_TO_DB_ERROR otherwise.
        """
        try:
            if self.is_skipped:
                return TestCase.EX_PUSH_TO_DB_ERROR
            assert self.project_name
            assert self.case_name
            assert self.start_time
            assert self.stop_time
            url = env.get('TEST_DB_URL')
            data = {"project_name": self.project_name,
                    "case_name": self.case_name,
                    "details": self.details}
            data["installer"] = env.get('INSTALLER_TYPE')
            data["scenario"] = env.get('DEPLOY_SCENARIO')
            data["pod_name"] = env.get('NODE_NAME')
            data["build_tag"] = env.get('BUILD_TAG')
            data["criteria"] = 'PASS' if self.is_successful(
                ) == TestCase.EX_OK else 'FAIL'
            data["start_date"] = datetime.fromtimestamp(
                self.start_time).strftime('%Y-%m-%d %H:%M:%S')
            data["stop_date"] = datetime.fromtimestamp(
                self.stop_time).strftime('%Y-%m-%d %H:%M:%S')
            try:
                data["version"] = re.search(
                    TestCase._job_name_rule,
                    env.get('BUILD_TAG')).group(2)
            except Exception:  # pylint: disable=broad-except
                data["version"] = "unknown"
            req = requests.post(
                url, data=json.dumps(data, sort_keys=True),
                headers=self.headers)
            req.raise_for_status()
            if urlparse(url).scheme != "file":
                # href must be postprocessed as OPNFV testapi is misconfigured
                # (localhost is returned)
                uid = re.sub(r'^.*/api/v1/results/*', '', req.json()["href"])
                netloc = env.get('TEST_DB_EXT_URL') if env.get(
                    'TEST_DB_EXT_URL') else env.get('TEST_DB_URL')
                self.__logger.info(
                    "The results were successfully pushed to DB: \n\n%s\n",
                    os.path.join(netloc, uid))
        except AssertionError:
            self.__logger.exception(
                "Please run test before publishing the results")
            return TestCase.EX_PUSH_TO_DB_ERROR
        except requests.exceptions.HTTPError:
            self.__logger.exception("The HTTP request raises issues")
            return TestCase.EX_PUSH_TO_DB_ERROR
        except Exception:  # pylint: disable=broad-except
            self.__logger.exception("The results cannot be pushed to DB")
            return TestCase.EX_PUSH_TO_DB_ERROR
        return TestCase.EX_OK

    def publish_artifacts(self):  # pylint: disable=too-many-locals
        """Push the artifacts to the S3 repository.

        It allows publishing the artifacts.

        It could be overriden if the common implementation is not
        suitable.

        The credentials must be configured before publishing the artifacts:

            * fill ~/.aws/credentials or ~/.boto,
            * set AWS_ACCESS_KEY_ID and AWS_SECRET_ACCESS_KEY in env.

        The next vars must be set in env:

            * S3_ENDPOINT_URL (http://127.0.0.1:9000),
            * S3_DST_URL (s3://xtesting/prefix),
            * HTTP_DST_URL (http://127.0.0.1/prefix).

        Returns:
            TestCase.EX_OK if artifacts were published to repository.
            TestCase.EX_PUBLISH_ARTIFACTS_ERROR otherwise.
        """
        try:
            b3resource = boto3.resource(
                's3', endpoint_url=os.environ["S3_ENDPOINT_URL"])
            dst_s3_url = os.environ["S3_DST_URL"]
            multipart_threshold = 5 * 1024 ** 5 if "google" in os.environ[
                "S3_ENDPOINT_URL"] else 8 * 1024 * 1024
            config = TransferConfig(multipart_threshold=multipart_threshold)
            bucket_name = urlparse(dst_s3_url).netloc
            try:
                b3resource.meta.client.head_bucket(Bucket=bucket_name)
            except botocore.exceptions.ClientError as exc:
                error_code = exc.response['Error']['Code']
                if error_code == '404':
                    # pylint: disable=no-member
                    b3resource.create_bucket(Bucket=bucket_name)
                else:
                    raise exc
            except Exception as exc:  # pylint: disable=broad-except
                raise exc
            path = urlparse(dst_s3_url).path.strip("/")
            dst_http_url = os.environ["HTTP_DST_URL"]
            output_str = "\n"
            self.details["links"] = []
            for log_file in [self.output_log_name, self.output_debug_log_name]:
                if os.path.exists(os.path.join(self.dir_results, log_file)):
                    abs_file = os.path.join(self.dir_results, log_file)
                    mime_type = mimetypes.guess_type(abs_file)
                    self.__logger.debug(
                        "Publishing %s %s", abs_file, mime_type)
                    # pylint: disable=no-member
                    b3resource.Bucket(bucket_name).upload_file(
                        abs_file, os.path.join(path, log_file), Config=config,
                        ExtraArgs={'ContentType': mime_type[
                            0] or 'application/octet-stream'})
                    link = os.path.join(dst_http_url, log_file)
                    output_str += f"\n{link}"
                    self.details["links"].append(link)
            for root, _, files in os.walk(self.res_dir):
                for pub_file in files:
                    abs_file = os.path.join(root, pub_file)
                    mime_type = mimetypes.guess_type(abs_file)
                    self.__logger.debug(
                        "Publishing %s %s", abs_file, mime_type)
                    # pylint: disable=no-member
                    b3resource.Bucket(bucket_name).upload_file(
                        abs_file,
                        os.path.join(path, os.path.relpath(
                            os.path.join(root, pub_file),
                            start=self.dir_results)),
                        Config=config,
                        ExtraArgs={'ContentType': mime_type[
                            0] or 'application/octet-stream'})
                    link = os.path.join(dst_http_url, os.path.relpath(
                        os.path.join(root, pub_file),
                        start=self.dir_results))
                    output_str += f"\n{link}"
                    self.details["links"].append(link)
            self.__logger.info(
                "All artifacts were successfully published: %s\n", output_str)
            return TestCase.EX_OK
        except KeyError as ex:
            self.__logger.error("Please check env var: %s", str(ex))
            return TestCase.EX_PUBLISH_ARTIFACTS_ERROR
        except botocore.exceptions.NoCredentialsError:
            self.__logger.error(
                "Please fill ~/.aws/credentials, ~/.boto or set "
                "AWS_ACCESS_KEY_ID and AWS_SECRET_ACCESS_KEY in env")
            return TestCase.EX_PUBLISH_ARTIFACTS_ERROR
        except Exception:  # pylint: disable=broad-except
            self.__logger.exception("Cannot publish the artifacts")
            return TestCase.EX_PUBLISH_ARTIFACTS_ERROR

    def clean(self):
        """Clean the resources.

        It can be overriden if resources must be deleted after
        running the test case.
        """



#!/usr/bin/env python

# Copyright (c) 2017 Orange and others.
#
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Apache License, Version 2.0
# which accompanies this distribution, and is available at
# http://www.apache.org/licenses/LICENSE-2.0

# pylint: disable=missing-docstring

import errno
import functools
import os
from urllib.parse import urlparse

import mock
import requests.sessions


def can_dump_request_to_file(method):

    def dump_preparedrequest(request, **kwargs):
        # pylint: disable=unused-argument
        parseresult = urlparse(request.url)
        if parseresult.scheme == "file":
            try:
                dirname = os.path.dirname(parseresult.path)
                os.makedirs(dirname)
            except OSError as ex:
                if ex.errno != errno.EEXIST:
                    raise
            with open(parseresult.path, 'a', encoding='utf-8') as dumpfile:
                headers = ""
                for key in request.headers:
                    headers += key + " " + request.headers[key] + "\n"
                message = (
                    f"{request.method} {request.url}"
                    f"\n{headers}\n{request.body}\n\n\n")
                dumpfile.write(message)
        return mock.Mock()

    def patch_request(method, url, **kwargs):
        with requests.sessions.Session() as session:
            parseresult = urlparse(url)
            if parseresult.scheme == "file":
                with mock.patch.object(session, 'send',
                                       side_effect=dump_preparedrequest):
                    return session.request(method=method, url=url, **kwargs)
            else:
                return session.request(method=method, url=url, **kwargs)

    @functools.wraps(method)
    def hook(*args, **kwargs):
        with mock.patch('requests.api.request', side_effect=patch_request):
            return method(*args, **kwargs)

    return hook



#!/usr/bin/env python

# pylint: disable=missing-docstring

import os

ENV_FILE = '/var/lib/xtesting/conf/env_file'

RESULTS_DIR = '/var/lib/xtesting/results'
LOG_PATH = os.path.join(RESULTS_DIR, 'xtesting.log')
DEBUG_LOG_PATH = os.path.join(RESULTS_DIR, 'xtesting.debug.log')

INI_PATH = 'ci/logging.ini'
DEBUG_INI_PATH = 'ci/logging.debug.ini'



#!/usr/bin/env python

# Copyright (c) 2016 Orange and others.
#
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Apache License, Version 2.0
# which accompanies this distribution, and is available at
# http://www.apache.org/licenses/LICENSE-2.0

# pylint: disable=missing-docstring

import unittest


class TestStringMethods(unittest.TestCase):

    def test_upper(self):
        self.assertEqual('Hello World'.upper(),
                         'HELLO WORLD')



#!/usr/bin/env python

# Copyright (c) 2019 Orange and others.
#
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Apache License, Version 2.0
# which accompanies this distribution, and is available at
# http://www.apache.org/licenses/LICENSE-2.0

"""Define the classes required to fully cover behave."""

import logging
import os
import unittest

import mock

from xtesting.core import behaveframework

__author__ = "Deepak Chandella <deepak.chandella@orange.com>"


class ParseResultTesting(unittest.TestCase):

    """The class testing BehaveFramework.parse_results()."""
    # pylint: disable=missing-docstring

    _response = [{'status': 'passed'}]

    def setUp(self):
        self.test = behaveframework.BehaveFramework(
            case_name='behave', project_name='xtesting')

    @mock.patch('builtins.open', side_effect=OSError)
    def test_raises_exc_open(self, *args):  # pylint: disable=unused-argument
        with self.assertRaises(OSError):
            self.test.parse_results()

    @mock.patch('json.load', return_value=[{'foo': 'bar'}])
    @mock.patch('builtins.open', mock.mock_open())
    def test_raises_exc_key(self, *args):  # pylint: disable=unused-argument
        with self.assertRaises(KeyError):
            self.test.parse_results()

    @mock.patch('json.load', return_value=[])
    @mock.patch('builtins.open', mock.mock_open())
    def test_raises_exe_zerodivision(self, *args):
        # pylint: disable=unused-argument
        with self.assertRaises(ZeroDivisionError):
            self.test.parse_results()

    def _test_result(self, response, result):
        with mock.patch('builtins.open', mock.mock_open()), \
                mock.patch('json.load', return_value=response):
            self.test.parse_results()
            self.assertEqual(self.test.result, result)

    def test_null_passed(self):
        data = [{'status': 'dummy'}]
        self._test_result(data, 0)

    def test_half_success(self):
        data = [{'status': 'passed'}, {'status': 'failed'}]
        self._test_result(data, 50)

    def test_success(self):
        data = [{'status': 'passed'}, {'status': 'passed'}]
        self._test_result(data, 100)

    @mock.patch('builtins.open', mock.mock_open())
    def test_count(self, *args):  # pylint: disable=unused-argument
        self._response.extend([{'status': 'failed'}, {'status': 'skipped'}])
        with mock.patch('json.load', mock.Mock(return_value=self._response)):
            self.test.parse_results()
            self.assertEqual(self.test.details['pass_tests'], 1)
            self.assertEqual(self.test.details['fail_tests'], 1)
            self.assertEqual(self.test.details['skip_tests'], 1)
            self.assertEqual(self.test.details['total_tests'], 3)


class RunTesting(unittest.TestCase):

    """The class testing BehaveFramework.run()."""
    # pylint: disable=missing-docstring

    suites = ["foo"]
    tags = ["bar"]

    def setUp(self):
        self.test = behaveframework.BehaveFramework(
            case_name='behave', project_name='xtesting')

    def test_exc_key_error(self):
        self.assertEqual(self.test.run(), self.test.EX_RUN_ERROR)

    @mock.patch('xtesting.core.behaveframework.behave_main')
    def _test_makedirs_exc(self, *args):
        with mock.patch.object(self.test, 'parse_results') as mock_method:
            self.assertEqual(
                self.test.run(
                    suites=self.suites, tags=self.tags),
                self.test.EX_RUN_ERROR)
            args[0].assert_not_called()
            mock_method.assert_not_called()

    @mock.patch('os.makedirs', side_effect=Exception)
    @mock.patch('os.path.exists', return_value=False)
    def test_makedirs_exc(self, *args):
        self._test_makedirs_exc()
        args[0].assert_called_once_with(self.test.res_dir)
        args[1].assert_called_once_with(self.test.res_dir)

    @mock.patch('xtesting.core.behaveframework.behave_main')
    def _test_makedirs(self, *args):
        with mock.patch.object(self.test, 'parse_results') as mock_method:
            self.assertEqual(
                self.test.run(suites=self.suites, tags=self.tags),
                self.test.EX_OK)
            html_file = os.path.join(self.test.res_dir, 'output.html')
            args_list = [
                '--junit',
                f'--junit-directory={self.test.res_dir}',
                '--format=json', f'--outfile={self.test.json_file}',
                '--format=behave_html_formatter:HTMLFormatter',
                f'--outfile={html_file}',
                '--tags='+','.join(self.tags)]
            args_list.append('foo')
            args[0].assert_called_once_with(args_list)
            mock_method.assert_called_once_with()

    @mock.patch('os.makedirs')
    @mock.patch('os.path.exists', return_value=False)
    def test_makedirs(self, *args):
        self._test_makedirs()
        args[0].assert_called_once_with(self.test.res_dir)
        args[1].assert_called_once_with(self.test.res_dir)

    @mock.patch('os.makedirs')
    @mock.patch('os.path.exists', return_value=True)
    def test_makedirs_oserror17(self, *args):
        self._test_makedirs()
        args[0].assert_called_once_with(self.test.res_dir)
        args[1].assert_not_called()

    @mock.patch('os.makedirs')
    @mock.patch('xtesting.core.behaveframework.behave_main')
    def _test_parse_results(self, status, console, *args):
        self.assertEqual(
            self.test.run(
                suites=self.suites, tags=self.tags, console=console),
            status)
        html_file = os.path.join(self.test.res_dir, 'output.html')
        args_list = [
            '--junit',
            f'--junit-directory={self.test.res_dir}',
            '--format=json', f'--outfile={self.test.json_file}',
            '--format=behave_html_formatter:HTMLFormatter',
            f'--outfile={html_file}',
            '--tags='+','.join(self.tags)]
        if console:
            args_list += ['--format=pretty', '--outfile=-']
        args_list.append('foo')
        args[0].assert_called_once_with(args_list)
        args[1].assert_called_once_with(self.test.res_dir)

    def test_parse_results_exc(self, console=False):
        with mock.patch.object(self.test, 'parse_results',
                               side_effect=Exception) as mock_method:
            self._test_parse_results(self.test.EX_RUN_ERROR, console)
            mock_method.assert_called_once_with()

    def test_parse_results_exc_console(self):
        self.test_parse_results_exc(console=True)


if __name__ == "__main__":
    logging.disable(logging.CRITICAL)
    unittest.main(verbosity=2)



#!/usr/bin/env python

# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Apache License, Version 2.0
# which accompanies this distribution, and is available at
# http://www.apache.org/licenses/LICENSE-2.0

# pylint: disable=missing-docstring

import logging
import unittest
import os

import mock

from xtesting.ci import run_tests
from xtesting.core.testcase import TestCase


class FakeModule(TestCase):

    def run(self, **kwargs):
        self.start_time = 0
        self.stop_time = 1
        self.criteria = 100
        self.result = 100
        return TestCase.EX_OK


class RunTestsTesting(unittest.TestCase):

    def setUp(self):
        self.runner = run_tests.Runner()
        mock_test_case = mock.Mock()
        mock_test_case.is_successful.return_value = TestCase.EX_OK
        self.runner.executed_test_cases['test1'] = mock_test_case
        self.runner.executed_test_cases['test2'] = mock_test_case
        self.sep = 'test_sep'
        self.creds = {'OS_AUTH_URL': 'http://test_ip:test_port/v2.0',
                      'OS_USERNAME': 'test_os_username',
                      'OS_TENANT_NAME': 'test_tenant',
                      'OS_PASSWORD': 'test_password'}
        self.test = {'test_name': 'test_name'}
        self.tier = mock.Mock()
        test1 = mock.Mock()
        test1.get_name.return_value = 'test1'
        test2 = mock.Mock()
        test2.get_name.return_value = 'test2'
        attrs = {'get_name.return_value': 'test_tier',
                 'get_tests.return_value': [test1, test2],
                 'get_ci_loop.return_value': 'test_ci_loop',
                 'get_test_names.return_value': ['test1', 'test2']}
        self.tier.configure_mock(**attrs)

        self.tiers = mock.Mock()
        attrs = {'get_tiers.return_value': [self.tier]}
        self.tiers.configure_mock(**attrs)

        self.run_tests_parser = run_tests.RunTestsParser()

    @mock.patch('xtesting.ci.run_tests.Runner.get_dict_by_test')
    def test_get_run_dict(self, *args):
        retval = {'run': mock.Mock()}
        args[0].return_value = retval
        self.assertEqual(self.runner.get_run_dict('test_name'), retval['run'])
        args[0].assert_called_once_with('test_name')

    @mock.patch('xtesting.ci.run_tests.LOGGER.error')
    @mock.patch('xtesting.ci.run_tests.Runner.get_dict_by_test',
                return_value=None)
    def test_get_run_dict_config_ko(self, *args):
        testname = 'test_name'
        self.assertEqual(self.runner.get_run_dict(testname), None)
        args[0].return_value = {}
        self.assertEqual(self.runner.get_run_dict(testname), None)
        calls = [mock.call(testname), mock.call(testname)]
        args[0].assert_has_calls(calls)
        calls = [mock.call("Cannot get %s's config options", testname),
                 mock.call("Cannot get %s's config options", testname)]
        args[1].assert_has_calls(calls)

    @mock.patch('xtesting.ci.run_tests.LOGGER.exception')
    @mock.patch('xtesting.ci.run_tests.Runner.get_dict_by_test',
                side_effect=Exception)
    def test_get_run_dict_exception(self, *args):
        testname = 'test_name'
        self.assertEqual(self.runner.get_run_dict(testname), None)
        args[1].assert_called_once_with(
            "Cannot get %s's config options", testname)

    def _test_source_envfile(self, msg, key='OS_TENANT_NAME', value='admin'):
        try:
            del os.environ[key]
        except Exception:  # pylint: disable=broad-except
            pass
        envfile = 'rc_file'
        with mock.patch('builtins.open',
                        mock.mock_open(read_data=msg)) as mock_method,\
                mock.patch('os.path.isfile', return_value=True):
            mock_method.return_value.__iter__ = lambda self: iter(
                self.readline, '')
            self.runner.source_envfile(envfile)
            mock_method.assert_called_once_with(envfile, 'r', encoding='utf-8')
            self.assertEqual(os.environ[key], value)

    def test_source_envfile(self):
        self._test_source_envfile('OS_TENANT_NAME=admin')
        self._test_source_envfile('OS_TENANT_NAME= admin')
        self._test_source_envfile('OS_TENANT_NAME = admin')
        self._test_source_envfile('OS_TENANT_NAME = "admin"')
        self._test_source_envfile('export OS_TENANT_NAME=admin')
        self._test_source_envfile('export OS_TENANT_NAME =admin')
        self._test_source_envfile('export OS_TENANT_NAME = admin')
        self._test_source_envfile('export OS_TENANT_NAME = "admin"')
        # This test will fail as soon as rc_file is fixed
        self._test_source_envfile(
            'export "\'OS_TENANT_NAME\'" = "\'admin\'"')

    def test_get_dict_by_test(self):
        with mock.patch('builtins.open', mock.mock_open()), \
                mock.patch('yaml.safe_load') as mock_yaml:
            mock_obj = mock.Mock()
            testcase_dict = {'case_name': 'testname',
                             'criteria': 50}
            attrs = {'get.return_value': [{'testcases': [testcase_dict]}]}
            mock_obj.configure_mock(**attrs)
            mock_yaml.return_value = mock_obj
            self.assertDictEqual(
                run_tests.Runner.get_dict_by_test('testname'),
                testcase_dict)

    @mock.patch('xtesting.ci.run_tests.Runner.get_run_dict',
                return_value=None)
    def test_run_tests_import_exception(self, *args):
        mock_test = mock.Mock()
        kwargs = {'get_name.return_value': 'test_name',
                  'is_skipped.return_value': False,
                  'is_enabled.return_value': True,
                  'needs_clean.return_value': False}
        mock_test.configure_mock(**kwargs)
        with self.assertRaises(Exception) as context:
            self.runner.run_test(mock_test)
        args[0].assert_called_with('test_name')
        msg = "Cannot import the class for the test case."
        self.assertTrue(msg in str(context.exception))

    @mock.patch('stevedore.driver.DriverManager',
                return_value=mock.Mock(driver=FakeModule()))
    @mock.patch('xtesting.ci.run_tests.Runner.get_dict_by_test')
    def test_run_tests_default(self, *args):
        mock_test = mock.Mock()
        kwargs = {'get_name.return_value': 'test_name',
                  'is_skipped.return_value': False,
                  'is_enabled.return_value': True,
                  'needs_clean.return_value': True}
        mock_test.configure_mock(**kwargs)
        test_run_dict = {'name': 'test_module'}
        with mock.patch('xtesting.ci.run_tests.Runner.get_run_dict',
                        return_value=test_run_dict):
            self.runner.clean_flag = True
            self.assertEqual(self.runner.run_test(mock_test), TestCase.EX_OK)
        args[0].assert_called_with('test_name')
        args[1].assert_called_with(
            invoke_kwds=mock.ANY, invoke_on_load=True, name='test_module',
            namespace='xtesting.testcase')
        self.assertEqual(self.runner.overall_result,
                         run_tests.Result.EX_OK)

    @mock.patch('xtesting.ci.run_tests.Runner.get_dict_by_test')
    def test_run_tests_disabled(self, *args):
        mock_test = mock.Mock()
        kwargs = {'get_name.return_value': 'test_name',
                  'is_skipped.return_value': False,
                  'is_enabled.return_value': False,
                  'needs_clean.return_value': True}
        mock_test.configure_mock(**kwargs)
        test_run_dict = {'name': 'test_name'}
        with mock.patch('xtesting.ci.run_tests.Runner.get_run_dict',
                        return_value=test_run_dict):
            self.runner.clean_flag = True
            self.runner.run_test(mock_test)
        args[0].assert_not_called()
        self.assertEqual(self.runner.overall_result,
                         run_tests.Result.EX_OK)

    @mock.patch('xtesting.ci.run_tests.Runner.get_dict_by_test')
    def test_run_tests_skipped(self, *args):
        mock_test = mock.Mock()
        kwargs = {'get_name.return_value': 'test_name',
                  'is_skipped.return_value': True,
                  'is_enabled.return_value': True,
                  'needs_clean.return_value': True}
        mock_test.configure_mock(**kwargs)
        test_run_dict = {'module': 'test_module',
                         'class': 'test_class'}
        with mock.patch('xtesting.ci.run_tests.Runner.get_run_dict',
                        return_value=test_run_dict):
            self.runner.clean_flag = True
            self.runner.run_test(mock_test)
        args[0].assert_not_called()
        self.assertEqual(self.runner.overall_result,
                         run_tests.Result.EX_OK)

    @mock.patch('xtesting.ci.run_tests.Runner.run_test',
                return_value=TestCase.EX_OK)
    def test_run_tier_default(self, *mock_methods):
        self.assertEqual(self.runner.run_tier(self.tier),
                         run_tests.Result.EX_OK)
        mock_methods[0].assert_called_with(mock.ANY)

    @mock.patch('xtesting.ci.run_tests.LOGGER.info')
    def test_run_tier_missing_test(self, mock_logger_info):
        self.tier.get_tests.return_value = None
        self.assertEqual(self.runner.run_tier(self.tier),
                         run_tests.Result.EX_ERROR)
        self.assertTrue(mock_logger_info.called)

    @mock.patch('xtesting.ci.run_tests.LOGGER.info')
    @mock.patch('xtesting.ci.run_tests.Runner.run_tier')
    @mock.patch('xtesting.ci.run_tests.Runner.summary')
    def test_run_all_default(self, *mock_methods):
        os.environ['CI_LOOP'] = 'test_ci_loop'
        self.runner.run_all()
        self.assertTrue(mock_methods[2].called)

    @mock.patch('xtesting.ci.run_tests.Runner.source_envfile',
                side_effect=Exception)
    @mock.patch('xtesting.ci.run_tests.Runner.summary')
    def test_main_failed(self, *mock_methods):
        kwargs = {'test': 'test_name', 'noclean': True, 'report': True}
        args = {'get_tier.return_value': False,
                'get_test.return_value': False}
        self.runner.tiers = mock.Mock()
        self.runner.tiers.configure_mock(**args)
        self.assertEqual(self.runner.main(**kwargs),
                         run_tests.Result.EX_ERROR)
        mock_methods[1].assert_called_once_with()

    @mock.patch('xtesting.ci.run_tests.Runner.source_envfile')
    @mock.patch('xtesting.ci.run_tests.Runner.run_test',
                return_value=TestCase.EX_OK)
    @mock.patch('xtesting.ci.run_tests.Runner.summary')
    def test_main_tier(self, *mock_methods):
        mock_tier = mock.Mock()
        test_mock = mock.Mock()
        test_mock.get_name.return_value = 'test1'
        args = {'get_name.return_value': 'tier_name',
                'get_tests.return_value': [test_mock]}
        mock_tier.configure_mock(**args)
        kwargs = {'test': 'tier_name', 'noclean': True, 'report': True}
        args = {'get_tier.return_value': mock_tier,
                'get_test.return_value': None}
        self.runner.tiers = mock.Mock()
        self.runner.tiers.configure_mock(**args)
        self.assertEqual(self.runner.main(**kwargs),
                         run_tests.Result.EX_OK)
        mock_methods[1].assert_called()

    @mock.patch('xtesting.ci.run_tests.Runner.source_envfile')
    @mock.patch('xtesting.ci.run_tests.Runner.run_test',
                return_value=TestCase.EX_OK)
    def test_main_test(self, *mock_methods):
        kwargs = {'test': 'test_name', 'noclean': True, 'report': True}
        args = {'get_tier.return_value': None,
                'get_test.return_value': 'test_name'}
        self.runner.tiers = mock.Mock()
        mock_methods[1].return_value = self.creds
        self.runner.tiers.configure_mock(**args)
        self.assertEqual(self.runner.main(**kwargs),
                         run_tests.Result.EX_OK)
        mock_methods[0].assert_called_once_with('test_name')

    @mock.patch('xtesting.ci.run_tests.Runner.source_envfile')
    @mock.patch('xtesting.ci.run_tests.Runner.run_all')
    @mock.patch('xtesting.ci.run_tests.Runner.summary')
    def test_main_all_tier(self, *args):
        kwargs = {'get_tier.return_value': None,
                  'get_test.return_value': None}
        self.runner.tiers = mock.Mock()
        self.runner.tiers.configure_mock(**kwargs)
        self.assertEqual(
            self.runner.main(test='all', noclean=True, report=True),
            run_tests.Result.EX_OK)
        args[0].assert_called_once_with(None)
        args[1].assert_called_once_with()
        args[2].assert_called_once_with()

    @mock.patch('xtesting.ci.run_tests.Runner.source_envfile')
    def test_main_any_tier_test_ko(self, *args):
        kwargs = {'get_tier.return_value': None,
                  'get_test.return_value': None}
        self.runner.tiers = mock.Mock()
        self.runner.tiers.configure_mock(**kwargs)
        self.assertEqual(
            self.runner.main(test='any', noclean=True, report=True),
            run_tests.Result.EX_ERROR)
        args[0].assert_called_once_with()


if __name__ == "__main__":
    logging.disable(logging.CRITICAL)
    unittest.main(verbosity=2)



#!/usr/bin/env python

# Copyright (c) 2016 Orange and others.
#
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Apache License, Version 2.0
# which accompanies this distribution, and is available at
# http://www.apache.org/licenses/LICENSE-2.0

# pylint: disable=missing-docstring

"""Define the class required to fully cover testcase."""

from datetime import datetime
import json
import logging
import os
import unittest

import botocore
import mock
import requests

from xtesting.core import testcase
from xtesting.utils import constants

__author__ = "Cedric Ollivier <cedric.ollivier@orange.com>"


class FakeTestCase(testcase.TestCase):
    # pylint: disable=too-many-instance-attributes

    def run(self, **kwargs):
        return testcase.TestCase.EX_OK


class AbstractTestCaseTesting(unittest.TestCase):

    def test_run_unimplemented(self):
        # pylint: disable=abstract-class-instantiated
        with self.assertRaises(TypeError):
            testcase.TestCase(case_name="base", project_name="xtesting")


class TestCaseTesting(unittest.TestCase):
    # pylint: disable=too-many-instance-attributes,too-many-public-methods

    _case_name = "base"
    _project_name = "xtesting"
    _published_result = "PASS"
    _test_db_url = "http://testresults.opnfv.org/test/api/v1/results"
    _headers = {'Content-Type': 'application/json'}

    def setUp(self):
        self.test = FakeTestCase(
            case_name=self._case_name, project_name=self._project_name)
        self.test.start_time = 1
        self.test.stop_time = 2
        self.test.result = 100
        self.test.details = {"Hello": "World"}
        os.environ['TEST_DB_URL'] = TestCaseTesting._test_db_url
        os.environ['TEST_DB_EXT_URL'] = TestCaseTesting._test_db_url
        os.environ['INSTALLER_TYPE'] = "installer_type"
        os.environ['DEPLOY_SCENARIO'] = "scenario"
        os.environ['NODE_NAME'] = "node_name"
        os.environ['BUILD_TAG'] = "foo-daily-master-bar"
        os.environ['S3_ENDPOINT_URL'] = "http://127.0.0.1:9000"
        os.environ['S3_DST_URL'] = "s3://xtesting/prefix"
        os.environ['HTTP_DST_URL'] = "http://127.0.0.1/prefix"

    def test_run_fake(self):
        self.assertEqual(self.test.run(), testcase.TestCase.EX_OK)

    def _test_pushdb_missing_attribute(self):
        self.assertEqual(self.test.push_to_db(),
                         testcase.TestCase.EX_PUSH_TO_DB_ERROR)

    def test_pushdb_no_project_name(self):
        self.test.project_name = None
        self._test_pushdb_missing_attribute()

    def test_pushdb_no_case_name(self):
        self.test.case_name = None
        self._test_pushdb_missing_attribute()

    def test_pushdb_no_start_time(self):
        self.test.start_time = None
        self._test_pushdb_missing_attribute()

    def test_pushdb_no_stop_time(self):
        self.test.stop_time = None
        self._test_pushdb_missing_attribute()

    def _test_pushdb_missing_env(self, var):
        del os.environ[var]
        self.assertEqual(self.test.push_to_db(),
                         testcase.TestCase.EX_PUSH_TO_DB_ERROR)

    def test_pushdb_no_db_url(self):
        self._test_pushdb_missing_env('TEST_DB_URL')

    def test_pushdb_no_installer_type(self):
        self._test_pushdb_missing_env('INSTALLER_TYPE')

    def test_pushdb_no_deploy_scenario(self):
        self._test_pushdb_missing_env('DEPLOY_SCENARIO')

    def test_pushdb_no_node_name(self):
        self._test_pushdb_missing_env('NODE_NAME')

    def test_pushdb_no_build_tag(self):
        self._test_pushdb_missing_env('BUILD_TAG')

    @mock.patch('requests.post')
    def test_pushdb_bad_start_time(self, mock_function=None):
        self.test.start_time = "1"
        self.assertEqual(
            self.test.push_to_db(),
            testcase.TestCase.EX_PUSH_TO_DB_ERROR)
        mock_function.assert_not_called()

    @mock.patch('requests.post')
    def test_pushdb_bad_end_time(self, mock_function=None):
        self.test.stop_time = "2"
        self.assertEqual(
            self.test.push_to_db(),
            testcase.TestCase.EX_PUSH_TO_DB_ERROR)
        mock_function.assert_not_called()

    @mock.patch('requests.post')
    def test_pushdb_skipped_test(self, mock_function=None):
        self.test.is_skipped = True
        self.assertEqual(
            self.test.push_to_db(),
            testcase.TestCase.EX_PUSH_TO_DB_ERROR)
        mock_function.assert_not_called()

    def _get_data(self):
        return {
            "build_tag": os.environ['BUILD_TAG'],
            "case_name": self._case_name,
            "criteria": 'PASS' if self.test.is_successful(
                ) == self.test.EX_OK else 'FAIL',
            "details": self.test.details,
            "installer": os.environ['INSTALLER_TYPE'],
            "pod_name": os.environ['NODE_NAME'],
            "project_name": self.test.project_name,
            "scenario": os.environ['DEPLOY_SCENARIO'],
            "start_date": datetime.fromtimestamp(
                self.test.start_time).strftime('%Y-%m-%d %H:%M:%S'),
            "stop_date": datetime.fromtimestamp(
                self.test.stop_time).strftime('%Y-%m-%d %H:%M:%S'),
            "version": "master"}

    @mock.patch('os.path.join', return_value='')
    @mock.patch('re.sub', return_value='')
    @mock.patch('requests.post')
    def _test_pushdb_version(self, *args, **kwargs):
        payload = self._get_data()
        payload["version"] = kwargs.get("version", "unknown")
        self.assertEqual(self.test.push_to_db(), testcase.TestCase.EX_OK)
        args[0].assert_called_once_with(
            os.environ['TEST_DB_URL'],
            data=json.dumps(payload, sort_keys=True),
            headers=self._headers)

    def test_pushdb_daily_job(self):
        self._test_pushdb_version(version="master")

    def test_pushdb_weekly_job(self):
        os.environ['BUILD_TAG'] = 'foo-weekly-master-bar'
        self._test_pushdb_version(version="master")

    def test_pushdb_random_build_tag(self):
        os.environ['BUILD_TAG'] = 'whatever'
        self._test_pushdb_version(version="unknown")

    @mock.patch('requests.post', return_value=mock.Mock(
        raise_for_status=mock.Mock(
            side_effect=requests.exceptions.HTTPError)))
    def test_pushdb_http_errors(self, mock_function=None):
        self.assertEqual(
            self.test.push_to_db(),
            testcase.TestCase.EX_PUSH_TO_DB_ERROR)
        mock_function.assert_called_once_with(
            os.environ['TEST_DB_URL'],
            data=json.dumps(self._get_data(), sort_keys=True),
            headers=self._headers)

    def test_check_requirements(self):
        self.test.check_requirements()
        self.assertEqual(self.test.is_skipped, False)

    def test_check_criteria_missing(self):
        self.test.criteria = None
        self.assertEqual(self.test.is_successful(),
                         testcase.TestCase.EX_TESTCASE_FAILED)

    def test_check_result_missing(self):
        self.test.result = None
        self.assertEqual(self.test.is_successful(),
                         testcase.TestCase.EX_TESTCASE_FAILED)

    def test_check_result_failed(self):
        # Backward compatibility
        # It must be removed as soon as TestCase subclasses
        # stop setting result = 'PASS' or 'FAIL'.
        self.test.result = 'FAIL'
        self.assertEqual(self.test.is_successful(),
                         testcase.TestCase.EX_TESTCASE_FAILED)

    def test_check_result_pass(self):
        # Backward compatibility
        # It must be removed as soon as TestCase subclasses
        # stop setting result = 'PASS' or 'FAIL'.
        self.test.result = 'PASS'
        self.assertEqual(self.test.is_successful(),
                         testcase.TestCase.EX_OK)

    def test_check_result_skip(self):
        self.test.is_skipped = True
        self.assertEqual(self.test.is_successful(),
                         testcase.TestCase.EX_TESTCASE_SKIPPED)

    def test_check_result_lt(self):
        self.test.result = 50
        self.assertEqual(self.test.is_successful(),
                         testcase.TestCase.EX_TESTCASE_FAILED)

    def test_check_result_eq(self):
        self.test.result = 100
        self.assertEqual(self.test.is_successful(),
                         testcase.TestCase.EX_OK)

    def test_check_result_gt(self):
        self.test.criteria = 50
        self.test.result = 100
        self.assertEqual(self.test.is_successful(),
                         testcase.TestCase.EX_OK)

    def test_check_result_zero(self):
        self.test.criteria = 0
        self.test.result = 0
        self.assertEqual(self.test.is_successful(),
                         testcase.TestCase.EX_TESTCASE_FAILED)

    def test_get_duration_start_ko(self):
        self.test.start_time = None
        self.assertEqual(self.test.get_duration(), "XX:XX")
        self.test.start_time = 0
        self.assertEqual(self.test.get_duration(), "XX:XX")

    def test_get_duration_end_ko(self):
        self.test.stop_time = None
        self.assertEqual(self.test.get_duration(), "XX:XX")
        self.test.stop_time = 0
        self.assertEqual(self.test.get_duration(), "XX:XX")

    def test_get_invalid_duration(self):
        self.test.start_time = 2
        self.test.stop_time = 1
        self.assertEqual(self.test.get_duration(), "XX:XX")

    def test_get_zero_duration(self):
        self.test.start_time = 2
        self.test.stop_time = 2
        self.assertEqual(self.test.get_duration(), "00:00")

    def test_get_duration(self):
        self.test.start_time = 1
        self.test.stop_time = 180
        self.assertEqual(self.test.get_duration(), "02:59")

    def test_get_duration_skipped_test(self):
        self.test.is_skipped = True
        self.assertEqual(self.test.get_duration(), "00:00")

    def test_str_project_name_ko(self):
        self.test.project_name = None
        self.assertIn("FakeTestCase object at", str(self.test))

    def test_str_case_name_ko(self):
        self.test.case_name = None
        self.assertIn("FakeTestCase object at", str(self.test))

    def test_str_pass(self):
        duration = '01:01'
        with mock.patch.object(self.test, 'get_duration',
                               return_value=duration), \
                mock.patch.object(self.test, 'is_successful',
                                  return_value=testcase.TestCase.EX_OK):
            message = str(self.test)
        self.assertIn(self._project_name, message)
        self.assertIn(self._case_name, message)
        self.assertIn(duration, message)
        self.assertIn('PASS', message)

    def test_str_fail(self):
        duration = '00:59'
        with mock.patch.object(self.test, 'get_duration',
                               return_value=duration), \
                mock.patch.object(
                    self.test, 'is_successful',
                    return_value=testcase.TestCase.EX_TESTCASE_FAILED):
            message = str(self.test)
        self.assertIn(self._project_name, message)
        self.assertIn(self._case_name, message)
        self.assertIn(duration, message)
        self.assertIn('FAIL', message)

    def test_str_skip(self):
        self.test.is_skipped = True
        message = str(self.test)
        self.assertIn(self._project_name, message)
        self.assertIn(self._case_name, message)
        self.assertIn("00:00", message)
        self.assertIn('SKIP', message)

    def test_clean(self):
        self.assertEqual(self.test.clean(), None)

    def _test_publish_artifacts_nokw(self, key):
        del os.environ[key]
        self.assertEqual(self.test.publish_artifacts(),
                         testcase.TestCase.EX_PUBLISH_ARTIFACTS_ERROR)

    def test_publish_artifacts_exc1(self):
        for key in ["S3_ENDPOINT_URL", "S3_DST_URL", "HTTP_DST_URL"]:
            self._test_publish_artifacts_nokw(key)

    @mock.patch('boto3.resource',
                side_effect=botocore.exceptions.NoCredentialsError)
    def test_publish_artifacts_exc2(self, *args):
        self.assertEqual(self.test.publish_artifacts(),
                         testcase.TestCase.EX_PUBLISH_ARTIFACTS_ERROR)
        args[0].assert_called_once_with(
            's3', endpoint_url=os.environ['S3_ENDPOINT_URL'])

    @mock.patch('boto3.resource', side_effect=Exception)
    def test_publish_artifacts_exc3(self, *args):
        self.assertEqual(self.test.publish_artifacts(),
                         testcase.TestCase.EX_PUBLISH_ARTIFACTS_ERROR)
        args[0].assert_called_once_with(
            's3', endpoint_url=os.environ['S3_ENDPOINT_URL'])

    @mock.patch('boto3.resource')
    def test_publish_artifacts_exc4(self, *args):
        args[0].return_value.meta.client.head_bucket.side_effect = Exception
        self.assertEqual(self.test.publish_artifacts(),
                         testcase.TestCase.EX_PUBLISH_ARTIFACTS_ERROR)
        args[0].assert_called_once_with(
            's3', endpoint_url=os.environ['S3_ENDPOINT_URL'])

    @mock.patch('boto3.resource')
    def test_publish_artifacts_exc5(self, *args):
        error_response = {'Error': {'Code': '403'}}
        mock_head_bucket = args[0].return_value.meta.client.head_bucket
        mock_head_bucket.side_effect = botocore.exceptions.ClientError(
            error_response, 'Foo')
        self.assertEqual(self.test.publish_artifacts(),
                         testcase.TestCase.EX_PUBLISH_ARTIFACTS_ERROR)
        args[0].assert_called_once_with(
            's3', endpoint_url=os.environ['S3_ENDPOINT_URL'])

    @mock.patch('mimetypes.guess_type', return_value=(None, None))
    @mock.patch('boto3.resource')
    @mock.patch('os.walk', return_value=[])
    def test_publish_artifacts1(self, *args):
        self.assertEqual(self.test.publish_artifacts(),
                         testcase.TestCase.EX_OK)
        args[0].assert_called_once_with(self.test.res_dir)
        args[1].assert_called_once_with(
            's3', endpoint_url=os.environ['S3_ENDPOINT_URL'])

    @mock.patch('mimetypes.guess_type', return_value=(None, None))
    @mock.patch('boto3.resource')
    @mock.patch('os.walk', return_value=[])
    def test_publish_artifacts2(self, *args):
        error_response = {'Error': {'Code': '404'}}
        mock_head_bucket = args[1].return_value.meta.client.head_bucket
        mock_head_bucket.side_effect = botocore.exceptions.ClientError(
            error_response, 'NoSuchBucket')
        self.assertEqual(self.test.publish_artifacts(),
                         testcase.TestCase.EX_OK)
        args[0].assert_called_once_with(self.test.res_dir)
        args[1].assert_called_once_with(
            's3', endpoint_url=os.environ['S3_ENDPOINT_URL'])

    @mock.patch('mimetypes.guess_type', return_value=(None, None))
    @mock.patch('os.path.exists', return_value=True)
    @mock.patch('boto3.resource')
    @mock.patch('os.walk',
                return_value=[
                    (testcase.TestCase.dir_results, ('',), ('bar',))])
    def test_publish_artifacts3(self, *args):
        self.assertEqual(self.test.publish_artifacts(),
                         testcase.TestCase.EX_OK)
        args[0].assert_called_once_with(self.test.res_dir)
        expected = [
            mock.call('s3', endpoint_url=os.environ['S3_ENDPOINT_URL']),
            mock.call().meta.client.head_bucket(Bucket='xtesting'),
            mock.call().Bucket('xtesting'),
            mock.call().Bucket().upload_file(
                constants.LOG_PATH,
                os.path.join('prefix', os.path.basename(constants.LOG_PATH)),
                Config=mock.ANY,
                ExtraArgs={'ContentType': 'application/octet-stream'}),
            mock.call().Bucket('xtesting'),
            mock.call().Bucket().upload_file(
                constants.DEBUG_LOG_PATH,
                os.path.join('prefix',
                             os.path.basename(constants.DEBUG_LOG_PATH)),
                Config=mock.ANY,
                ExtraArgs={'ContentType': 'application/octet-stream'}),
            mock.call().Bucket('xtesting'),
            mock.call().Bucket().upload_file(
                os.path.join(constants.RESULTS_DIR, 'bar'),
                'prefix/bar',
                Config=mock.ANY,
                ExtraArgs={'ContentType': 'application/octet-stream'})]
        self.assertEqual(args[1].mock_calls, expected)

    @mock.patch('mimetypes.guess_type', return_value=('text/plain', None))
    @mock.patch('os.path.exists', return_value=True)
    @mock.patch('boto3.resource')
    @mock.patch('os.walk',
                return_value=[
                    (testcase.TestCase.dir_results, ('',), ('bar',))])
    def test_publish_artifacts4(self, *args):
        self.assertEqual(self.test.publish_artifacts(),
                         testcase.TestCase.EX_OK)
        args[0].assert_called_once_with(self.test.res_dir)
        expected = [
            mock.call('s3', endpoint_url=os.environ['S3_ENDPOINT_URL']),
            mock.call().meta.client.head_bucket(Bucket='xtesting'),
            mock.call().Bucket('xtesting'),
            mock.call().Bucket().upload_file(
                constants.LOG_PATH,
                os.path.join('prefix', os.path.basename(constants.LOG_PATH)),
                Config=mock.ANY,
                ExtraArgs={'ContentType': 'text/plain'}),
            mock.call().Bucket('xtesting'),
            mock.call().Bucket().upload_file(
                constants.DEBUG_LOG_PATH,
                os.path.join('prefix',
                             os.path.basename(constants.DEBUG_LOG_PATH)),
                Config=mock.ANY,
                ExtraArgs={'ContentType': 'text/plain'}),
            mock.call().Bucket('xtesting'),
            mock.call().Bucket().upload_file(
                os.path.join(constants.RESULTS_DIR, 'bar'),
                'prefix/bar',
                Config=mock.ANY,
                ExtraArgs={'ContentType': 'text/plain'})]
        self.assertEqual(args[1].mock_calls, expected)


if __name__ == "__main__":
    logging.disable(logging.CRITICAL)
    unittest.main(verbosity=2)



#!/usr/bin/env python

# Copyright (c) 2016 Orange and others.
#
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Apache License, Version 2.0
# which accompanies this distribution, and is available at
# http://www.apache.org/licenses/LICENSE-2.0

# pylint: disable=missing-docstring

import logging
import unittest

import mock

from xtesting.core import vnf
from xtesting.core import testcase


class VnfBaseTesting(unittest.TestCase):
    """The class testing VNF."""
    # pylint: disable=missing-docstring,too-many-public-methods

    tenant_name = 'test_tenant_name'
    tenant_description = 'description'

    def setUp(self):
        self.test = vnf.VnfOnBoarding(project='xtesting', case_name='foo')

    def test_run_deploy_orch_exc(self):
        with mock.patch.object(self.test, 'prepare'), \
                mock.patch.object(self.test, 'deploy_orchestrator',
                                  side_effect=Exception) as mock_method, \
                mock.patch.object(self.test, 'deploy_vnf',
                                  return_value=True), \
                mock.patch.object(self.test, 'test_vnf',
                                  return_value=True):
            self.assertEqual(self.test.run(),
                             testcase.TestCase.EX_TESTCASE_FAILED)
            mock_method.assert_called_with()

    def test_run_deploy_vnf_exc(self):
        with mock.patch.object(self.test, 'prepare'),\
            mock.patch.object(self.test, 'deploy_orchestrator',
                              return_value=True), \
            mock.patch.object(self.test, 'deploy_vnf',
                              side_effect=Exception) as mock_method:
            self.assertEqual(self.test.run(),
                             testcase.TestCase.EX_TESTCASE_FAILED)
            mock_method.assert_called_with()

    def test_run_test_vnf_exc(self):
        with mock.patch.object(self.test, 'prepare'),\
            mock.patch.object(self.test, 'deploy_orchestrator',
                              return_value=True), \
            mock.patch.object(self.test, 'deploy_vnf', return_value=True), \
            mock.patch.object(self.test, 'test_vnf',
                              side_effect=Exception) as mock_method:
            self.assertEqual(self.test.run(),
                             testcase.TestCase.EX_TESTCASE_FAILED)
            mock_method.assert_called_with()

    def test_run_deploy_orch_ko(self):
        with mock.patch.object(self.test, 'prepare'),\
                mock.patch.object(self.test, 'deploy_orchestrator',
                                  return_value=False), \
                mock.patch.object(self.test, 'deploy_vnf',
                                  return_value=True), \
                mock.patch.object(self.test, 'test_vnf',
                                  return_value=True):
            self.assertEqual(self.test.run(),
                             testcase.TestCase.EX_TESTCASE_FAILED)

    def test_run_vnf_deploy_ko(self):
        with mock.patch.object(self.test, 'prepare'),\
                mock.patch.object(self.test, 'deploy_orchestrator',
                                  return_value=True), \
                mock.patch.object(self.test, 'deploy_vnf',
                                  return_value=False), \
                mock.patch.object(self.test, 'test_vnf',
                                  return_value=True):
            self.assertEqual(self.test.run(),
                             testcase.TestCase.EX_TESTCASE_FAILED)

    def test_run_vnf_test_ko(self):
        with mock.patch.object(self.test, 'prepare'),\
                mock.patch.object(self.test, 'deploy_orchestrator',
                                  return_value=True), \
                mock.patch.object(self.test, 'deploy_vnf',
                                  return_value=True), \
                mock.patch.object(self.test, 'test_vnf',
                                  return_value=False):
            self.assertEqual(self.test.run(),
                             testcase.TestCase.EX_TESTCASE_FAILED)

    def test_run_default(self):
        with mock.patch.object(self.test, 'prepare'),\
                mock.patch.object(self.test, 'deploy_orchestrator',
                                  return_value=True), \
                mock.patch.object(self.test, 'deploy_vnf',
                                  return_value=True), \
                mock.patch.object(self.test, 'test_vnf',
                                  return_value=True):
            self.assertEqual(self.test.run(), testcase.TestCase.EX_OK)

    def test_prepare(self):
        with self.assertRaises(vnf.VnfPreparationException):
            self.test.prepare()

    def test_deploy_vnf_unimplemented(self):
        with self.assertRaises(vnf.VnfDeploymentException):
            self.test.deploy_vnf()

    def test_test_vnf_unimplemented(self):
        with self.assertRaises(vnf.VnfTestException):
            self.test.test_vnf()

    def test_deploy_orch_unimplemented(self):
        self.assertTrue(self.test.deploy_orchestrator())

    def test_vnf_clean_exc(self):
        obj = mock.Mock()
        obj.clean.side_effect = Exception
        self.test.created_object = [obj]
        self.test.clean()
        obj.clean.assert_called_with()

    def test_vnf_clean(self):
        obj = mock.Mock()
        self.test.created_object = [obj]
        self.test.clean()
        obj.clean.assert_called_with()


if __name__ == "__main__":
    logging.disable(logging.CRITICAL)
    unittest.main(verbosity=2)
